// WAR_LOOT.JS
//
// Author:	Poundboy
// Version:	2.08
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  09/18/03  1.04	Initial version.
//
//
// Description:
//
// Contains all constants and functions related to loot and salvage processing.
//

LootDebug		= false;
LootDebug2		= false;
LootDebugScreen		= false;

lootTypeJunk		=  0;
lootTypeItem		=  1;
lootTypeRare		=  2;
lootTypeMajor		=  3;
lootTypeMinor		=  4;
lootTypeArmor		=  5;
lootTypeWeapon		=  6;
lootTypeJewelry		=  7;
lootTypeClothing	=  8;
lootTypeWand		=  9;
lootTypeScroll		= 10;
lootTypeSalvage		= 11;
lootTypeValSalv		= 12
lootTypeValuable	= 13;
lootTypeMax		= 13;
lootTypeTotal		= 14;

// Define beginning and ending ranges of good loot
lootTypeBegin		= 1;
lootTypeEnd		= 13;

// Combine point constants for salvaging
combineMaskVal = new Array();
combineMaskVal[1]	= 0x00000001;
combineMaskVal[2]	= 0x00000002;
combineMaskVal[3]	= 0x00000004;
combineMaskVal[4]	= 0x00000008;
combineMaskVal[5]	= 0x00000010;
combineMaskVal[6]	= 0x00000020;
combineMaskVal[7]	= 0x00000040;
combineMaskVal[8]	= 0x00000080;
combineMaskVal[9]	= 0x00000100;
combineMaskVal[10]	= 0x00000200;

// Material salvage type
salTypeImbue	= 1;
salTypeNonImbue	= 2;
salTypeUnknown	= 3;

salTypeText = new Array();
salTypeText[salTypeImbue]	= "Imbue";
salTypeText[salTypeNonImbue]	= "NonImbue";
salTypeText[salTypeUnknown]	= "Unknown";

// Wand andor codes
andorFlagText = new Array();
andorFlagText[0] = "AND";
andorFlagText[1] = "OR";

armorShield	= eqmShield;
armorHead	= eqmHead;
armorHands	= eqmHands;
armorFeet	= eqmFeet;
armorBody	= eqmChestOuter | eqmAbdomenOuter | eqmUpperArmsOuter |
		  eqmLowerArmsOuter | eqmUpperLegsOuter | eqmLowerLegsOuter;
armorExtremity	= armorHead | armorHands | armorFeet;
armorPerson	= armorBody | armorExtremity;
armorAll	= armorPerson | armorShield;

// Pick up items with the following names.  Thanks to David A. Down (Merlyn of Harvestion) for most of the
// following list (from AutoLoot)
lootItemList = new Array(
		"Pyreal Pea",
		"Gold Pea",
		"Silver Pea",
		"Copper Pea",
		"Iron Pea",
		"Lead Pea",
		"Pyreal Scarab",
		"Gold Scarab",
		"Silver Scarab",
		"Copper Scarab",
		"Iron Scarab",
		"Lead Scarab",
		"A Large Mnemosyne",
		"A Small Mnemosyne",
		"A Tiny Mnemosyne",
		"Ash Gromnie Tooth",
		"Aun Pendant",
		"Auroch Horn",
		"Banderling Scalp",
		"Basalt Blade",
		"Black Rat Tail",
		"Black Rock",
		"Black Stone",
		"Blue Gem",
		"Blue Phyntos Wasp Wing",
		"Broken Virindi Mask",
		"Broken Virindi Profatrix Mask",
		"Bronze Armeradillo Spine",
		"Bronze Armoredillo Spine",
		"Bronze Coil",
		"Bronze Gear from a Statue",
		"Bronze Gear",
		"Brown Beans",
		"Bunch of Nanners",
		"Canescent Mattekar Pelt",
		"Carenzi Burrower Pelt",
		"Carenzi Pouchling Pelt",
		"Carenzi Sentry Pelt",
		"Carenzi Stalker Pelt",
		"Caulnalain Key",
		"Copper Heart",
		"Coral Heart",
		"Coral Golem Heart",
		"Cracked Shard",
		"Dark Revenant Thighbone",
		"Dark Shard",
		"Dark Sliver",
		"Dark Speck",
		"Dark Tower",
		"Dark Towers",
		"Diamond Heart",
		"Diamond Powder",
		"Dire Mattakar Paw",
		"Doll Mask",
		"Dread Mattakar Paw",
		"Dread Mattekar Paw",
		"Drudge Charm",
		"Elaniwood Heart",
		"Eviscerator Head",
		"Fenmalain Key",
		"Fire Auroch Hide",
		"Fire Auroch Horn",
		"Glowing Wisp Heart",
		"Gold Phyntos Wasp Wing",
		"Granite Heart",
		"Great Mattekar Horn",
		"Green Phyntos Wasp Wing",
		"Grey Armeradillo Spine",
		"Grievver Tibia",
		"Gromnie Hide",
		"Hoary Armeradillo Spine",
		"Hoary Armoredillo Spine",
		"Hoary Mattakar Hide",
		"Hot Pepper",
		"Idol Gem",
		"Iron Heart",
		"Island Armeradillo Spine",
		"Ivory Gromnie Tooth",
		"Jade Gromnie Tooth",
		"Jungle Phyntos Wasp Wing",
		"Large Armoredillo Hide",
		"Large Lugian Sinew",
		"Large Sclavus Hide",
		"Large Shreth Hide",
		"Large Ursuin Hide",
		"Lich Skull",
		"Little Green Seeds",
		"Littoral Siraluun Claw",
		"Mattekar Hide",
		"Mire Phyntos Wasp Wing",
		"Monougat",
		"Niffis Shell",
		"Obsidian Heart",
		"Olthoi Claw",
		"Olthoi Head",
		"Olthoi Ichor",
		"Oubliette",
		"Peppermint Stick",
		"Plant",
		"Plate Armoredillo Spine",
		"Pulsating Wisp Heart",
		"Pyreal Mote",
		"Pyreal Nugget",
		"Pyreal Sliver",
		"Rabbit Carcass",
		"Rabid Carenzi Pelt",
		"Ravener Guts",
		"Red Jewel",
		"Reedshark Hide",
		"Sand Golem Heart",
		"Sandy Armoredillo Spine",
		"Sclavus Hide",
		"Seal",
		"Sharp Tusker Slave Tusk",
		"Shendolain Key",
		"Shore Armeradillo Spine",
		"Shreth Hide",
		"Skeleton's Skull",
		"Skull of High Acolyte",
		"Small Armoredillo Hide",
		"Small Bundle of Kithless Siraluun Feathers",
		"Small Lugian Sinew",
		"Small Niffis Shell",
		"Small Reedshark Hide",
		"Small Sclavus Hide",
		"Small Shard",
		"Small Ursuin Hide",
		"Strange Stick",
		"Sturdy Iron Key",
		"Sturdy Steel Key",
		"Swamp Gromnie Tooth",
		"Swamp Stone",
		"Thick Gromnie Hide",
		"Thin Gromnie Hide",
		"Timber Siraluun Claw",
		"Tiny Shard",
		"Tree Trunk",
		"Twisted Dark Key",
		"Undead Femur Bone",
		"Undead Femur bone",
		"Undead Thighbone",
		"Ursuin Fang",
		"Ursuin Scalp",
		"Vapor Heart",
		"Vial of Organic Acid",
		"Virindi Directive Key",
		"Virindi Essense",
		"Virindi Implant",
		"Virindi Master Key",
		"Virindi Singularity Key",
		"White Jewel",
		"White Phyntos Wasp Wing",
		"Wood Heart",
		"Yellow Jewel" );

		// "A Lucky Gold Letter",
		// "A Lump of Coal",
		// "A Scarlet Red Letter",

// Make a list of the Major and Minor Spells.  Yes I could just search through all possible spells
// but I want them in a certain order.
majorSpellList = new Array(
		"Major Coordination", "Major Endurance", "Major Focus",
		"Major Quickness", "Major Strength", "Major Willpower",
		"Major Creature Enchantment Aptitude", "Major Item Enchantment Aptitude",
		"Major Life Magic Aptitude", "Major War Magic Aptitude",
		"Major Magic Resistance", "Major Mana Conversion Prowess",
		"Major Axe Aptitude", "Major Bow Aptitude", "Major Crossbow Aptitude", "Major Dagger Aptitude",
		"Major Mace Aptitude", "Major Spear Aptitude", "Major Staff Aptitude", "Major Sword Aptitude",
		"Major Thrown Aptitude", "Major Unarmed Aptitude", "Major Impregnability", "Major Invulnerability",
		"Major Armor Tinkering Expertise", "Major Item Tinkering Expertise",
		"Major Magic Item Tinkering Expertise", "Major Weapon Tinkering Expertise",
		"Major Alchemical Prowess", "Major Cooking Prowess", "Major Fletching Prowess",
		"Major Arcane Prowess", "Major Deception Prowess", "Major Fealty", "Major Healing Prowess",
		"Major Jumping Prowess", "Major Leadership", "Major Lockpick Prowess", "Major Monster Attunement",
		"Major Person Attunement", "Major Sprint",
		"Major Blood Thirst", "Major Defender", "Major Heart Thirst", "Major Swift Hunter",
		"Major Armor", "Major Acid Ward", "Major Bludgeoning Ward", "Major Flame Ward",
		"Major Piercing Ward", "Major Slashing Ward", "Major Frost Ward", "Major Storm Ward",
		"Major Impenetrability", "Major Acid Bane", "Major Bludgeoning Bane", "Major Flame Bane",
		"Major Frost Bane", "Major Piercing Bane", "Major Slashing Bane", "Major Storm Bane" );

minorSpellList = new Array(
		"Minor Coordination", "Minor Endurance", "Minor Focus",
		"Minor Quickness", "Minor Strength", "Minor Willpower",
		"Minor Creature Enchantment Aptitude", "Minor Item Enchantment Aptitude",
		"Minor Life Magic Aptitude", "Minor War Magic Aptitude",
		"Minor Magic Resistance", "Minor Mana Conversion Prowess",
		"Minor Axe Aptitude", "Minor Bow Aptitude", "Minor Crossbow Aptitude", "Minor Dagger Aptitude",
		"Minor Mace Aptitude", "Minor Spear Aptitude", "Minor Staff Aptitude", "Minor Sword Aptitude",
		"Minor Thrown Aptitude", "Minor Unarmed Aptitude", "Minor Impregnability", "Minor Invulnerability",
		"Minor Armor Tinkering Expertise", "Minor Item Tinkering Expertise",
		"Minor Magic Item Tinkering Expertise", "Minor Weapon Tinkering Expertise",
		"Minor Alchemical Prowess", "Minor Cooking Prowess", "Minor Fletching Prowess",
		"Minor Arcane Prowess", "Minor Deception Prowess", "Minor Fealty", "Minor Healing Prowess",
		"Minor Jumping Prowess", "Minor Leadership", "Minor Lockpick Prowess", "Minor Monster Attunement",
		"Minor Person Attunement", "Minor Sprint",
		"Minor Blood Thirst", "Minor Defender", "Minor Heart Thirst", "Minor Swift Hunter",
		"Minor Armor", "Minor Acid Ward", "Minor Bludgeoning Ward", "Minor Flame Ward",
		"Minor Frost Ward", "Minor Piercing Ward", "Minor Slashing Ward", "Minor Storm Ward",
		"Minor Impenetrability", "Minor Acid Bane", "Minor Bludgeoning Bane", "Minor Flame Bane",
		"Minor Frost Bane", "Minor Piercing Bane", "Minor Slashing Bane", "Minor Storm Bane" );

ItemMajorPat	= new RegExp("(^.*Major.*$)", "g" );	// Used to detect majors
ItemMinorPat	= new RegExp("(^.*Minor.*$)", "g" );	// Used to detect minors
ItemMinorBanePat= new RegExp("(^Minor.*Bane$)", "g" );	// Used to detect minor banes
ItemMinorWardPat= new RegExp("(^Minor.*Ward$)", "g" );	// Used to detect minor wards


// Flags set when special markers detected in loot config file
LOOT_BOF	= false;
LOOT_EOF	= false;


function MaterialRangeRec(Low,High) {
    this.Low  = Low;
    this.High = High;
} // end object

// Define filter for each item type.  Note at the moment the Rare type does not have a filter
// rec as we pick up either all of them or none.
function ItemFilterRec(Name) {
    this.Name		= Name;
    this.Enabled 	= false;
    this.Alert		= false;
} // endif

function MajorFilterRec(Name,SpellId) {
    this.Name		= Name;
    this.SpellId	= SpellId;
    this.Enabled	= true;		// By default pickup all majors
} // endif

function MinorFilterRec(Name,SpellId) {
    this.Name		= Name;
    this.SpellId	= SpellId;
    this.Enabled	= false;
    this.Eqm		= 0;		// Minor must be on the specified body location
} // endif

function ArmorFilterRec(Name, Eqm, CovFlag) {
    // Fixed variables that identify an armor item
    this.Name		= Name;
    this.Eqm		= Eqm;
    this.CovFlag	= CovFlag;

    // Variables settable by user
    this.Enabled	= false;
    this.MinAL		= 0;
    this.MaxWork	= 0;
    this.MaxBU		= 0;
    this.MaxValue	= 0;
    this.MinSpells	= 0;
    this.MinSplLvl	= 0;
} // endif

// Object that defines a weapon's loot collection profile
function WeaponFilterRec(Name, Skid, DamTypes) {
    // Fixed variables that identify a weapon item
    this.Name		= Name;
    this.Skid		= Skid;
    this.DamTypes	= DamTypes;

    // Variables settable by user
    this.Enabled	= false;
    this.MinDam		= 0;
    this.MinAtt		= 0;
    this.MinDef		= 0;
    this.MaxWork	= 0;
    this.MaxBU		= 0;
    this.MaxValue	= 0;
    this.MinSpells	= 0;
    this.MinSplLvl	= 0;
} // end func

function WandFilterRec(Name) {
    // Variables settable by user.
    this.Name		= Name;
    this.Enabled	= false;
    this.MaxValue	= 0;
    this.MinManaC	= 0;
    this.MinMeleeD	= 0;
    this.AndOrFlag	= 0;
} // endif

function SalvageFilterRec(MaterialId, Type, Name, Desc) {
    // Fixed variables that identify a salvage type
    this.Name		= Name;
    this.MaterialId	= MaterialId;
    this.Type		= Type;
    this.Desc		= Desc;
    this.Symbol		= "material" + Name;

    // Private variable used in salvaging and constructed from CombineMask property
    this.WorkRange 	= new Array();	// For each workmanship level, holds the min and max work that will be
    					// will be combined into the same bag (uses MaterialRangeRec object)

    // Variables settable by user.
    this.Enabled	= false;
    this.ExcludeVal	= false;	// If set, then don't pick up this salvage type for selling purposes
    this.MinWork	= 0;		// Workmanship of loot item must be this before picking up.
    this.MaxBU		= 0;		// Maximum burden of loot. Item won't be picked up if burden > than this.
    this.SortPrio	= 0;		// Sort priority for each material.
    this.CombineMask	= combineMaskVal[10];
    					// Bit mask that defines the combine ranges.  Default is 1-10 which is done
} // end func				// by setting bit 10 and clearing all other bits.


function ValSalvFilterRec() {
    // Variables settable by user.
    this.MinRatio	= 0;		// Minimim value to work ratio before picking up item.  Ensures bags a good value
    this.MaxBagValue	= 100000;	// Maximum permitted value of salvage bag.  Needed because vendors don't accept
    					// really expensive bags.
} // endif

function JewelryFilterRec() {
    // Variables settable by user.
    this.MinSplLvl1	= 0;
    this.MinSplLvl2	= 0;
    this.MinSplLvl3	= 0;
} // endif

function ClothingFilterRec() {
    // Variables settable by user.
    this.MinSplLvl1	= 0;
    this.MinSplLvl2	= 0;
    this.MinSplLvl3	= 0;
} // endif

function ScrollFilterRec() {
    // Variables settable by user.
    this.MinLvl		= 7;
    this.MaxLvl		= 7;
    this.UnknownOnly	= false;
    this.LearnUnknown	= false;
} // endif

function ValuableFilterRec() {
    // Variables settable by user.
    this.MaxBU		= 0;
    this.MinValue	= 0;
    this.MaxTotal	= 0;
} // endif

function LootItemRec(Aco, Desc) {
    this.Oid  		= Aco.oid;	// oid of object.  Used to refresh aco if we logout/login
    this.Aco  		= Aco;		// Aco of loot object
    this.Desc 		= Desc;		// Detailed description of loot object generated from assess
    this.SalvageCount	= 0;		// # of times we tried to salvage this item if on salvage loot list
} // end func

function LootTypeRec(Name) {
    // Private properties
    this.Name		= Name;			// name of loot type
    this.Symbol		= "lootType" + Name;	// Symbolic name.  used when creating config files
    this.ItemList	= new Array();		// List of loot items (LootItemRec) of this loot type picked up
    this.ItemCount	= 0;			// Count of items on ItemList
    this.TotalValue	= 0;			// Total value of this loot type picked up

    // Properties settable by user
    this.Enabled	= true;			// If true, then pickup this loot type
    this.MaxCount	= 100;			// Maximum # of this loot type that may be picked up
    this.MinSlots	= 10;			// Minimum # of slots that must be free in main pack
    						// before we will pick up this loot type
 } // end function


function LootObj() {
    // Define some constants
    this.SalDefBur	= 250;			// Default maximum burden for salvage items

    // The following array defines one record for each loot type (weapons, armor, wands, salvage etc).
    this.TypeInfo	= new Array();    	// Holds LootTypeRec object.  Indexed by LootType* constants
    this.TypeCount	= lootTypeTotal;

    // The next set of variables holds the filters that define what we pick up.
    this.ItemFilter	= new Array();			// Items.  Array of ItemFilterRec
    this.MajorFilter	= new Array();			// Majors. Indexed by spell id. Array of MajorFilterRec
    this.MinorFilter	= new Array();			// Minors. Indexed by spell id. Array of MinorFilterRec
    this.ArmorFilter	= new Array();			// Armor. Array of ArmorFilterRec
    this.WeaponFilter	= new Array();			// Weapons (UA, sword, bow, etc). Array of WeaponFilterRec
    this.WandFilter	= new Array();			// Wands. Indexed by workmanship.  Array of WandFilterRec
    this.SalvageFilter	= new Array();			// Salvage.  Indexed by salvage id. Array of SalvageFilterRec
    this.ValSalvFilter  = new ValSalvFilterRec();	// Valuable salvage. Only one record needed.
    this.JewelryFilter	= new JewelryFilterRec();	// Jewelry. Only one record needed
    this.ClothingFilter	= new ClothingFilterRec();	// Clothing. Only one record needed
    this.ScrollFilter	= new ScrollFilterRec();	// Scrolls. Only one record needed
    this.ValuableFilter	= new ValuableFilterRec();	// Valuables (ie sellable stuff).  Only one record needed

    this.MajorIndex	= new Array();			// Translate spell id to MajorFilter offset
    this.MinorIndex	= new Array();			// Translate spell id to MinorFilter offset
    this.SalvageIndex	= new Array();			// Translate material id to SalvageFilter offset

    this.ItemOpts	= "";			// Decal option list of items
    this.MajorOpts	= "";			// Decal option list of majors (data value is spellid)
    this.MinorOpts	= "";			// Decal option list of minors (data value is spellid)
    this.ArmorOpts	= "";			// Decal option list of armor
    this.WeaponOpts	= "";			// Decal option list of weapons
    this.WandOpts	= "";			// Decal option list of wands
    this.SalvageOpt	= "";			// Decal option list of salvage (data value is material code)

    this.SpellCount	= 0;
    this.SpellFreq	= new Array();

    // Next two properties hold the result from the most recent loot search
    this.TypeIndex	= 0;
    this.ItemIndex	= 0;
    this.AlertFlag	= false;		// If the last item we called the WhatType method on was alertable
    						// then this flag is set.
    this.FileName	= "";			// Last filename built from a save or restore operation

    // New methods
    this.Init		= Loot_Init;		// Initialize loot structures
    this.LoadDefaults	= Loot_LoadDefaults;	// Set everything to default values
    this.Load		= Loot_Load;		// Load a profile from a configuration file
    this.Save		= Loot_Save;		// Save a profile to a configuration file

    this.EnableType	= Loot_EnableType;	//
    this.DisableType	= Loot_DisableType;	//
    this.SetType	= Loot_SetType;	 	//
    this.ModType	= Loot_ModType;	 	//
    this.ShowType	= Loot_ShowType;	//
    this.ResetType	= Loot_ResetType;	//
    this.WhatType	= Loot_WhatType;	//

    this.CheckArmor	= Loot_CheckArmor;
    this.CheckWeapon	= Loot_CheckWeapon;
    this.CheckMinSpellLvls
    			= Loot_CheckMinSpellLvls
    this.LookupLoot	= Loot_LookupLoot;	// Look for an OID on a loot list
    this.AddLoot	= Loot_AddLoot;		// Add an item to a loot list
    this.RemoveLoot	= Loot_RemoveLoot;	// Remove an item from a loot list
    this.RefreshLoot	= Loot_RefreshLoot;	// Refresh loot lists due to a logoff/login
    this.ResetLoot	= Loot_ResetLoot;	// Zero out loot lists

    this.EnableItem	= Loot_EnableItem;	//
    this.DisableItem	= Loot_DisableItem;	//
    this.SetItem	= Loot_SetItem;		//
    this.ModItem	= Loot_ModItem;
    this.LookupItem	= Loot_LookupItem;	// Lookup an item by name, returning its index
    this.ShowItem	= Loot_ShowItem;
    this.ResetItem	= Loot_ResetItem;

    this.EnableMajor	= Loot_EnableMajor;
    this.DisableMajor	= Loot_DisableMajor;
    this.SetMajor	= Loot_SetMajor;
    this.ShowMajor	= Loot_ShowMajor;
    this.ResetMajor	= Loot_ResetMajor;

    this.EnableMinor	= Loot_EnableMinor;
    this.DisableMinor	= Loot_DisableMinor;
    this.SetMinor	= Loot_SetMinor;
    this.ModMinor	= Loot_ModMinor;
    this.ShowMinor	= Loot_ShowMinor;
    this.ResetMinor	= Loot_ResetMinor;

    this.EnableArmor	= Loot_EnableArmor;	//
    this.DisableArmor	= Loot_DisableArmor;	//
    this.AddArmor	= Loot_AddArmor;	//
    this.SetArmor	= Loot_SetArmor;	//
    this.ModArmor	= Loot_ModArmor;	//
    this.ShowArmor	= Loot_ShowArmor;	//
    this.ResetArmor	= Loot_ResetArmor;	//
    this.LookupArmor	= Loot_LookupArmor;	// Lookup an armor type by name, returning its index

    this.EnableWeapon	= Loot_EnableWeapon;	// Enable a weapon to be picked up
    this.DisableWeapon	= Loot_DisableWeapon;	// Disable a weapon from pickup
    this.AddWeapon	= Loot_AddWeapon;	// Add a weapon to the weapon list
    this.SetWeapon	= Loot_SetWeapon;	// Set all weapon parameters
    this.ModWeapon	= Loot_ModWeapon;	// Set one or more weapon parameters
    this.ShowWeapon	= Loot_ShowWeapon;
    this.ResetWeapon	= Loot_ResetWeapon;
    this.LookupWeapon	= Loot_LookupWeapon;	// Lookup a weapon type by name, returning its index

    this.EnableWand	= Loot_EnableWand;
    this.DisableWand	= Loot_DisableWand;
    this.SetWand	= Loot_SetWand;
    this.SetWand2	= Loot_SetWand2;	// New paramters added for wands
    this.ModWand	= Loot_ModWand;
    this.ShowWand	= Loot_ShowWand;
    this.ResetWand	= Loot_ResetWand;

    this.EnableSalvage	= Loot_EnableSalvage;
    this.DisableSalvage	= Loot_DisableSalvage;
    this.AddSalvage	= Loot_AddSalvage;
    this.SetSalvage	= Loot_SetSalvage;
    this.ModSalvage	= Loot_ModSalvage;
    this.ShowSalvage	= Loot_ShowSalvage;
    this.ResetSalvage	= Loot_ResetSalvage;
    this.SetWorkRange	= Loot_SetWorkRange;

    this.ModValSalv	= Loot_ModValSalv;
    this.ShowValSalv	= Loot_ShowValSalv;
    this.ResetValSalv	= Loot_ResetValSalv;

    this.ModJewelry	= Loot_ModJewelry;
    this.ShowJewelry	= Loot_ShowJewelry;
    this.ResetJewelry	= Loot_ResetJewelry;

    this.ModClothing	= Loot_ModClothing;
    this.ShowClothing	= Loot_ShowClothing;
    this.ResetClothing	= Loot_ResetClothing;

    this.ModScroll	= Loot_ModScroll;
    this.ShowScroll	= Loot_ShowScroll;
    this.ResetScroll	= Loot_ResetScroll;

    this.ModValuable	= Loot_ModValuable;
    this.ShowValuable	= Loot_ShowValuable;
    this.ResetValuable	= Loot_ResetValuable;

} // end func


// CONFIGURATION METHODS **********************************************************

function Loot_Init(configFile) {
    if (LootDebug) Log.OutD("Loot_Init: Begin", LootDebugScreen);
    // Main LootType list
    this.TypeInfo[lootTypeJunk]		= new LootTypeRec("Junk");
    this.TypeInfo[lootTypeItem]		= new LootTypeRec("Item");
    this.TypeInfo[lootTypeRare]		= new LootTypeRec("Rare");
    this.TypeInfo[lootTypeMajor]	= new LootTypeRec("Major");
    this.TypeInfo[lootTypeMinor]	= new LootTypeRec("Minor");
    this.TypeInfo[lootTypeArmor]	= new LootTypeRec("Armor");
    this.TypeInfo[lootTypeWeapon]	= new LootTypeRec("Weapon");
    this.TypeInfo[lootTypeJewelry]	= new LootTypeRec("Jewelry");
    this.TypeInfo[lootTypeClothing]	= new LootTypeRec("Clothing");
    this.TypeInfo[lootTypeWand]		= new LootTypeRec("Wand");
    this.TypeInfo[lootTypeScroll]	= new LootTypeRec("Scroll");
    this.TypeInfo[lootTypeSalvage]	= new LootTypeRec("Salvage");
    this.TypeInfo[lootTypeValSalv]	= new LootTypeRec("ValSalv");
    this.TypeInfo[lootTypeValuable]	= new LootTypeRec("Valuable");

    // Items by name
    for (var x=0; x<lootItemList.length; x++)	{
    	var Name = lootItemList[x];
    	this.ItemFilter[x] = new ItemFilterRec(Name);
    	this.ItemOpts += " <option data='" + x + "' text=\"" + Name + "\" />\n";
    } // endfor

    // Majors
    for (var x=0; x<majorSpellList.length; x++)	{
    	var Name = majorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
    	this.MajorFilter[x] = new MajorFilterRec(Name,SpellId);
    	this.MajorIndex[SpellId] = x;
    	this.MajorOpts += " <option data='" + x + "' text='" + Name + "'/>\n";
    } // endfor

    // Minors
    for (var x=0; x<minorSpellList.length; x++)	{
   	var Name = minorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
    	this.MinorFilter[x] = new MinorFilterRec(Name,SpellId);
    	this.MinorIndex[SpellId] = x;
    	this.MinorOpts += " <option data='" + x + "' text='" + Name + "'/>\n";
    } // endfor

    // Armor
    this.AddArmor("Armor (Body)",	armorBody,	false);
    this.AddArmor("Armor (Head)",	armorHead, 	false);
    this.AddArmor("Armor (Hands)",	armorHands, 	false);
    this.AddArmor("Armor (Feet)",	armorFeet, 	false);
    this.AddArmor("Armor (Shield)",	armorShield, 	false);
    this.AddArmor("Cov Armor (Body)",	armorBody,	true );
    this.AddArmor("Cov Armor (Head)",	armorHead, 	true );
    this.AddArmor("Cov Armor (Hands)",	armorHands, 	true );
    this.AddArmor("Cov Armor (Feet)",	armorFeet, 	true );
    this.AddArmor("Cov Armor (Shield)",	armorShield, 	true );

    // Weapons
    this.AddWeapon("Bow",		skidBow,	    0				);
    this.AddWeapon("Crossbow",		skidCrossbow,	    0				);
    this.AddWeapon("Sword (Pierce)",	skidSword,	    dmtyPiercing		);
    this.AddWeapon("Sword (S/P)",	skidSword,	    dmtySlashing|dmtyPiercing 	);
    this.AddWeapon("Sword (Cold)",	skidSword,	    dmtyCold			);
    this.AddWeapon("Sword (Fire)",	skidSword, 	    dmtyFire 			);
    this.AddWeapon("Sword (Acid)",	skidSword, 	    dmtyAcid 			);
    this.AddWeapon("Sword (Elec)",	skidSword,	    dmtyElectrical		);
    this.AddWeapon("UA (Slash)",	skidUnarmedCombat,  dmtySlashing		);
    this.AddWeapon("UA (S/P)",		skidUnarmedCombat,  dmtySlashing|dmtyPiercing	);
    this.AddWeapon("UA (Bludg)",	skidUnarmedCombat,  dmtyBludgeoning 		);
    this.AddWeapon("UA (Cold)",		skidUnarmedCombat,  dmtyCold 			);
    this.AddWeapon("UA (Fire)",		skidUnarmedCombat,  dmtyFire 			);
    this.AddWeapon("UA (Acid)",		skidUnarmedCombat,  dmtyAcid 			);
    this.AddWeapon("UA (Elec)",		skidUnarmedCombat,  dmtyElectrical		);
    this.AddWeapon("Axe (Slash)",	skidAxe, 	    dmtySlashing 		);
    this.AddWeapon("Axe (Bludg)",	skidAxe, 	    dmtyBludgeoning 		);
    this.AddWeapon("Axe (Cold)",	skidAxe, 	    dmtyCold 			);
    this.AddWeapon("Axe (Fire)",	skidAxe, 	    dmtyFire 			);
    this.AddWeapon("Axe (Acid)",	skidAxe, 	    dmtyAcid 			);
    this.AddWeapon("Axe (Elec)",	skidAxe, 	    dmtyElectrical		);
    this.AddWeapon("Dagger (S/P)",	skidDagger, 	    dmtySlashing|dmtyPiercing	);
    this.AddWeapon("Dagger (Cold)",	skidDagger, 	    dmtyCold			);
    this.AddWeapon("Dagger (Fire)",	skidDagger, 	    dmtyFire			);
    this.AddWeapon("Dagger (Acid)",	skidDagger, 	    dmtyAcid			);
    this.AddWeapon("Dagger (Elec)",	skidDagger, 	    dmtyElectrical		);
    this.AddWeapon("Mace (Bludg)",	skidMace, 	    dmtyBludgeoning		);
    this.AddWeapon("Mace (Cold)",	skidMace, 	    dmtyCold			);
    this.AddWeapon("Mace (Fire)",	skidMace, 	    dmtyFire			);
    this.AddWeapon("Mace (Acid)",	skidMace, 	    dmtyAcid			);
    this.AddWeapon("Mace (Elec)",	skidMace, 	    dmtyElectrical		);
    this.AddWeapon("Spear (Pierce)",	skidSpear, 	    dmtyPiercing		);
    this.AddWeapon("Spear (S/P)",	skidSpear, 	    dmtySlashing|dmtyPiercing	);
    this.AddWeapon("Spear (Cold)",	skidSpear, 	    dmtyCold			);
    this.AddWeapon("Spear (Fire)",	skidSpear, 	    dmtyFire			);
    this.AddWeapon("Spear (Acid)",	skidSpear, 	    dmtyAcid			);
    this.AddWeapon("Spear (Elec)",	skidSpear, 	    dmtyElectrical		);
    this.AddWeapon("Staff (Bludg)",	skidStaff,	    dmtyBludgeoning		);
    this.AddWeapon("Staff (Cold)",	skidStaff, 	    dmtyCold			);
    this.AddWeapon("Staff (Fire)",	skidStaff, 	    dmtyFire			);
    this.AddWeapon("Staff (Acid)",	skidStaff, 	    dmtyAcid 			);
    this.AddWeapon("Staff (Elec)",	skidStaff,	    dmtyElectrical		);
    this.AddWeapon("Thrown Weapons",	skidThrownWeapons,  0				);

    // Wands
    for (var x=0; x<11; x++) {
    	var WandName = "Wand  (Work " + x + ")";
    	this.WandFilter[x] = new WandFilterRec(WandName);
    	if (x > 0) this.WandOpts += " <option data='" + x + "' text='" + WandName + "' />\n";
    } // end for

    this.AddSalvage(materialSteel,	    salTypeNonImbue,    "Steel",	 "Inc armor level by 20"	);
    this.AddSalvage(materialAlabaster,	    salTypeNonImbue,    "Alabaster",	 "Inc Pierce prot 0.2"		);
    this.AddSalvage(materialBronze,	    salTypeNonImbue,    "Bronze",	 "Inc Slash prot 0.2"		);
    this.AddSalvage(materialMarble,	    salTypeNonImbue,    "Marble",	 "Inc Bludg prot 0.2"		);
    this.AddSalvage(materialCeramic,	    salTypeNonImbue,    "Ceramic",	 "Inc Fire prot 0.4"		);
    this.AddSalvage(materialDilloHide,	    salTypeNonImbue,    "DilloHide",	 "Inc Acid prot 0.4"		);
    this.AddSalvage(materialReedsharkHide,  salTypeNonImbue,	"ReedsharkHide", "Inc Elec prot 0.4"		);
    this.AddSalvage(materialWool,	    salTypeNonImbue,    "Wool",		 "Inc Cold prot 0.4"		);
    this.AddSalvage(materialBrass,	    salTypeNonImbue,    "Brass",	 "Inc MeleeD Bonus by 1%"	);
    this.AddSalvage(materialGranite,	    salTypeNonImbue,    "Granite",	 "Dec Weapons Variance"		);
    this.AddSalvage(materialIron,	    salTypeNonImbue,    "Iron",		 "Inc Damage by 1"		);
    this.AddSalvage(materialMahogany,	    salTypeNonImbue,    "Mahogany",	 "Inc Dam Modifier by 4%"	);
    this.AddSalvage(materialOak,	    salTypeNonImbue,    "Oak",		 "Dec Speed by 5"		);
    this.AddSalvage(materialVelvet,	    salTypeNonImbue,    "Velvet",	 "Inc Attack Bonus by 1%"	);
    this.AddSalvage(materialGold,	    salTypeNonImbue,    "Gold",		 "Raise value by 25%"		);
    this.AddSalvage(materialLinen,	    salTypeNonImbue,    "Linen",	 "Reduce burden by 25%"		);
    this.AddSalvage(materialPine,	    salTypeNonImbue,    "Pine",		 "Reduce value by 25%"		);
    this.AddSalvage(materialMoonstone,	    salTypeNonImbue,    "Moonstone",	 "Inc maximum mana by 250"	);
    this.AddSalvage(materialIvory,	    salTypeImbue,       "Ivory",	 "Make item givable"		);
    this.AddSalvage(materialLeather,	    salTypeImbue,	"Leather",	 "Make item retained"		);
    this.AddSalvage(materialBlackOpal,	    salTypeImbue,	"BlackOpal",	 "Critical Strike Imbue"	);
    this.AddSalvage(materialFireOpal,	    salTypeImbue,	"FireOpal",	 "Crippling Blow Imbue"		);
    this.AddSalvage(materialSunstone,	    salTypeImbue,	"Sunstone",	 "Armor Rending Imbue"		);
    this.AddSalvage(materialOpal,	    salTypeNonImbue,	"Opal", 	 "Inc Mana Conv Mod by 1%"	);
    this.AddSalvage(materialWhiteSapphire,  salTypeImbue,	"WhiteSapphire", "Bludgeon Resist Render"	);
    this.AddSalvage(materialBlackGarnet,    salTypeImbue,	"BlackGarnet",	 "Pierce Resist Render"		);
    this.AddSalvage(materialImperialTopaz,  salTypeImbue,	"ImperialTopaz", "Slash Resist Render"		);
    this.AddSalvage(materialEmerald,	    salTypeImbue,	"Emerald",	 "Acid Resist Render"		);
    this.AddSalvage(materialRedGarnet,	    salTypeImbue,	"RedGarnet",	 "Fire Resist Render"		);
    this.AddSalvage(materialAquamarine,	    salTypeImbue,	"Aquamarine",	 "Frost Resist Render"		);
    this.AddSalvage(materialJet,	    salTypeImbue,	"Jet",		 "Lightning Resist Render"	);
    this.AddSalvage(materialEbony,	    salTypeNonImbue,	"Ebony",	 "Race to Ghar"			);
    this.AddSalvage(materialTeak,	    salTypeNonImbue,	"Teak", 	 "Race to Aluvian"		);
    this.AddSalvage(materialPorcelain,	    salTypeNonImbue,	"Porcelain",	 "Race to Sho"			);
    this.AddSalvage(materialSilver,	    salTypeNonImbue,	"Silver",	 "MeleeD to Missle"		);
    this.AddSalvage(materialCopper,	    salTypeNonImbue,	"Copper",	 "Missle to MeleeD"		);
    this.AddSalvage(materialSilk,	    salTypeNonImbue,	"Silk", 	 "Remove Rank"			);
    this.AddSalvage(materialYellowTopaz,    salTypeImbue,	"YellowTopaz",	 "Missle D +1"			);
    this.AddSalvage(materialPeridot,	    salTypeImbue,	"Peridot",	 "Melee D +1"			);
    this.AddSalvage(materialZircon,	    salTypeImbue,	"Zircon",	 "Magic D +1"			);
    this.AddSalvage(materialLavenderJade,   salTypeImbue,	"LavenderJade",	 "Minor Mana Gain"		);
    this.AddSalvage(materialRedJade,	    salTypeImbue,	"RedJade",	 "Minor Health Gain"		);
    this.AddSalvage(materialCitrine,	    salTypeImbue,	"Citrine",	 "Minor Stamina Gain"		);
    this.AddSalvage(materialCarnelian,	    salTypeImbue,	"Carnelian",	 "Minor Strength"		);
    this.AddSalvage(materialRoseQuartz,	    salTypeImbue,	"RoseQuartz",	 "Minor Quick"			);
    this.AddSalvage(materialBloodstone,	    salTypeImbue,	"Bloodstone",	 "Minor Endurance"		);
    this.AddSalvage(materialSmokeyQuartz,   salTypeImbue,	"SmokeyQuartz",	 "Minor Coord"			);
    this.AddSalvage(materialAgate,	    salTypeImbue,	"Agate",	 "Minor Focus"			);
    this.AddSalvage(materialLapisLazuli,    salTypeImbue,	"LapisLazuli",	 "Minor Self"			);
    this.AddSalvage(materialMalachite,	    salTypeImbue,	"Malachite",	 "Warrior's Vigor"		);
    this.AddSalvage(materialHematite,	    salTypeImbue,	"Hematite",	 "Warrior's Vitality"		);
    this.AddSalvage(materialAzurite,	    salTypeImbue,	"Azurite",	 "Wizard's Intellect"		);
    this.AddSalvage(materialAmber,	    salTypeUnknown,	"Amber",	 "Unknown use"			);
    this.AddSalvage(materialAmethyst,	    salTypeUnknown,	"Amethyst",	 "Unknown use"			);
    this.AddSalvage(materialDiamond,	    salTypeUnknown,	"Diamond",	 "Unknown use"			);
    this.AddSalvage(materialGreenGarnet,    salTypeUnknown,	"GreenGarnet",	 "Unknown use"			);
    this.AddSalvage(materialGreenJade,	    salTypeUnknown,	"GreenJade",	 "Unknown use"			);
    this.AddSalvage(materialGromnieHide,    salTypeUnknown,	"GromnieHide",	 "Unknown use"			);
    this.AddSalvage(materialObsidian,	    salTypeUnknown,	"Obsidian",	 "Unknown use"			);
    this.AddSalvage(materialOnyx,	    salTypeUnknown,	"Onyx", 	 "Unknown use"			);
    this.AddSalvage(materialPyreal,	    salTypeUnknown,	"Pyreal",	 "Unknown use"			);
    this.AddSalvage(materialRuby,	    salTypeUnknown,	"Ruby", 	 "Unknown use"			);
    this.AddSalvage(materialSandstone,	    salTypeUnknown,	"Sandstone",	 "Unknown use"			);
    this.AddSalvage(materialSapphire,	    salTypeUnknown,	"Sapphire",	 "Unknown use"			);
    this.AddSalvage(materialSatin,	    salTypeUnknown,	"Satin",	 "Unknown use"			);
    this.AddSalvage(materialSerpentine,     salTypeUnknown,	"Serpentine",	 "Unknown use"			);
    this.AddSalvage(materialTigerEye,	    salTypeUnknown,	"TigerEye",	 "Unknown use"			);
    this.AddSalvage(materialTourmaline,	    salTypeUnknown,	"Tourmaline",	 "Unknown use"			);
    this.AddSalvage(materialTurquoise,	    salTypeUnknown,	"Turquoise",	 "Unknown use"			);
    this.AddSalvage(materialWhiteJade,	    salTypeUnknown,	"WhiteJade",	 "Unknown use"			);
    this.AddSalvage(materialWhiteQuartz,    salTypeUnknown,	"WhiteQuartz",	 "Unknown use"			);
    this.AddSalvage(materialYellowGarnet,   salTypeUnknown,	"YellowGarnet",	 "Unknown use"			);

    if (LootDebug) Log.OutD("Loot_Init: End", LootDebugScreen);
} // end func

// Default values for loot structures
function Loot_LoadDefaults() {
    if (LootDebug) Log.OutD("Loot_LoadDefaults: Begin", LootDebugScreen);
    this.ResetType();		// Reset loot types
    this.ResetItem();		// Reset items
    this.ResetMajor();		// Reset majors
    this.ResetMinor();		// Reset minors to defaults
    this.ResetArmor();		// Reset armor
    this.ResetWeapon();		// Reset Weapons
    this.ResetSalvage();	// Reset salvage
    this.ResetWand();		// Reset wands
    this.ResetValSalv();	// Valuable salvage
    this.ResetJewelry();	// Reset jewelry
    this.ResetClothing();	// Reset clothing
    this.ResetScroll();		// Reset scroll
    this.ResetValuable();	// Reset valuable
    if (LootDebug) Log.OutD("Loot_LoadDefault: End", LootDebugScreen);
} // end func

function Loot_Save(fileNum) {
    if (LootDebug) LootLog.OutD("Loot_Save: Begin");

    // Build the output filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	var Post = "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    } // endif
    this.FileName = CharCfgDir + "\\" + fileLootCfg + Post;

    // Now create the file
    LootFile = Fso.OpenTextFile(this.FileName, ForWriting, true);	// Create config file

    // Write header record
    LootFile.WriteLine("// Loot config file created " + Util.FormatDate(0,false));
    LootFile.WriteLine("LOOT_BOF = true;")

    // First write type information.
    for (var x=1; x<this.TypeCount; x++) {
	with (this.TypeInfo[x]) {
	    LootFile.WriteLine("Loot.SetType( " + Symbol + ", " + Enabled + ", " + MaxCount + ", " + MinSlots + ");");
	} // end with
    } // end for

    // Write ItemFilter info.  Only need to write items that are enabled or we want to alert on since we set
    // everything disabled at macro start.
    for (var x=0; x<this.ItemFilter.length; x++) {
	with (this.ItemFilter[x]) {
	    if (Enabled || Alert) LootFile.WriteLine("Loot.SetItem(\"" + Name + "\", " + Enabled + ", " + Alert + ")");
	} // end with
    } // end for

    // Write MajorFilter info.  Only need to write items that are disabled since we enable all majors at startup
    for (var x=0; x<majorSpellList.length; x++) {
    	var Name = majorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
    	var Index = this.MajorIndex[SpellId];
	with (this.MajorFilter[Index]) {
	    if (!Enabled) LootFile.WriteLine("Loot.SetMajor( '" + Name + "', " + Enabled + ")");
	} // end with
    } // end for

    // Write MinorFilter info. Only need to write items that are enabled or have non-zero eqm
    for (var x=0; x<minorSpellList.length; x++) {
   	var Name = minorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
   	var Index = this.MinorIndex[SpellId];
	with (this.MinorFilter[Index]) {
	    if (Enabled || (Eqm > 0))
		LootFile.WriteLine("Loot.SetMinor( '" + Name + "', " + Enabled + ", " + Util.Hex(Eqm) + ")");
	} // end with
    } // end for

    // Write ArmorFilter info
    for (var x=0; x<this.ArmorFilter.length; x++) {
	with (this.ArmorFilter[x]) {
	    LootFile.WriteLine("Loot.SetArmor( '" + Name + "', " + Enabled + ", " + MinAL + ", " +  MaxWork + ", " +
	    			MaxBU + ", " + MaxValue + ", " + MinSpells + ", " + MinSplLvl + ")");
	} // end with
    } // end for

    // Write WeaponFilter info
   for (var x=0; x<this.WeaponFilter.length; x++) {
	with (this.WeaponFilter[x]) {
	    LootFile.WriteLine("Loot.SetWeapon( '" + Name + "', " + Enabled + ", " + MinDam + ", " +  MinAtt + ", " +
	    			MinDef + ", " + MaxWork + ", " + MaxBU + ", " + MaxValue + ", " + MinSpells + ", " +
	    			MinSplLvl + ")");
	} // end with
    } // end for

    // Write WandFilter info
    for (var x=1; x<11; x++) {
	with (this.WandFilter[x]) {
	    LootFile.WriteLine("Loot.SetWand2( " + x + ", " + Enabled + ", " + MaxValue + ", " + MinManaC + ", " + 
	    					MinMeleeD + ", " + AndOrFlag + ")");
	} // end with
    } // end for

    // Write SalvageFilter info
    for (var x=0; x<=100; x++) {
        if ((this.SalvageFilter[x] != undefined) && (this.SalvageFilter[x] != null)) {
	    with (this.SalvageFilter[x]) {
		LootFile.WriteLine("Loot.SetSalvage( " + Symbol + ", " + Enabled + ", " + MinWork + ", " + MaxBU +
				   ", " + SortPrio + ", " + Util.Hex(CombineMask) + ", " + ExcludeVal + ")");
	    } // end with
        } // endif
    } // end for

    // Write ValSalvFilter info
    with (this.ValSalvFilter)
	LootFile.WriteLine("Loot.ModValSalv( " + MinRatio + ", " +  MaxBagValue + ")" );

    // Write JewelryFilter info
    with (this.JewelryFilter)
	LootFile.WriteLine("Loot.ModJewelry( " + MinSplLvl1 + ", " +  MinSplLvl2 + ", " + MinSplLvl3 + ")" );

    // Write ClothingFilter info
    with (this.ClothingFilter)
	LootFile.WriteLine("Loot.ModClothing( " + MinSplLvl1 + ", " + MinSplLvl2 + ", " + MinSplLvl3 + ")" );

    // Write ScrollFilter info
    with (this.ScrollFilter)
	LootFile.WriteLine("Loot.ModScroll( " + MinLvl + ", " + MaxLvl + ", " + UnknownOnly + ", " + LearnUnknown + ")" );

    // Write ValuableFilter info
    with (this.ValuableFilter)
	LootFile.WriteLine("Loot.ModValuable( " + MaxBU + ", " + MinValue + ", " + MaxTotal + ")" );

    // All done, write EOF flag and close file
    LootFile.WriteLine("LOOT_EOF = true;")
    LootFile.Close();
    if (LootDebug) Log.OutD("Loot_Save: End", LootDebugScreen);
    return true;
} // endif

function Loot_Load(fileNum) {
    if (LootDebug) Log.OutD("Loot_Load: Begin", LootDebugScreen);

    // Build the output filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	var Post = "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    } // endif
    this.FileName = CharCfgDir + "\\" + fileLootCfg + Post;

    // See if file exists
    if(!Fso.FileExists(this.FileName)) return false;

    // Flags set from loot config file.  If not set then we have a problem with the config file
    LOOT_BOF = false;
    LOOT_EOF = false;

    // Read the config file
    Util.Include(this.FileName);

    if (!LOOT_EOF) {
	Chat.Screen("WARNING!!! End of file marker not detected in loot config file!", cmcYellow);
	Chat.Screen("WARNING!!! Loot file is either corrupt or did not write correctly last save operation!", cmcYellow);
	Chat.Screen("WARNING!!! Make sure your loot profiles are setup correctly!", cmcYellow);
    } // endif

    if (LootDebug) Log.OutD("Loot_Load: End", LootDebugScreen);
    return true;
} // endif


// LOOT TYPE METHODS ***********************************************************

function Loot_EnableType(TypeIndex) {
    this.TypeInfo[TypeIndex].Enabled = true;
} // end func

function Loot_DisableType(TypeIndex) {
    this.TypeInfo[TypeIndex].Enabled = false;
} // end func

function Loot_SetType(TypeIndex, Enabled, MaxCount, MinSlots) {
    this.TypeInfo[TypeIndex].Enabled  = Enabled;
    this.TypeInfo[TypeIndex].MaxCount = MaxCount;
    this.TypeInfo[TypeIndex].MinSlots = MinSlots;
} // end func

function Loot_ModType(TypeIndex, MaxCount, MinSlots) {
    if ((MaxCount == undefined) || (MaxCount == null)) MaxCount = -1;
    if ((MinSlots == undefined) || (MinSlots == null)) MinSlots = -1;
    if (MaxCount >= 0) this.TypeInfo[TypeIndex].MaxCount = MaxCount;
    if (MinSlots >= 0) this.TypeInfo[TypeIndex].MinSlots = MinSlots;
} // end func

function Loot_ResetType() {
   // Loot types:   SetType(type, pickup_flag, max_pickups, min_freeslots)
    this.SetType(lootTypeItem,		true,	100,	24 );
    this.SetType(lootTypeRare,		true,	100,	 0 );
    this.SetType(lootTypeMajor,		true,	100,	 0 );
    this.SetType(lootTypeMinor,		true,	100,	 4 );
    this.SetType(lootTypeArmor,		true,	100,	 8 );
    this.SetType(lootTypeWeapon,	true,	100,	 8 );
    this.SetType(lootTypeJewelry,	true,	100,	 8 );
    this.SetType(lootTypeClothing,	true,	100,	 8 );
    this.SetType(lootTypeWand,		true,	100,	 8 );
    this.SetType(lootTypeScroll,	true,	100,	 8 );
    this.SetType(lootTypeSalvage,	true,	100,	12 );
    this.SetType(lootTypeValuable,	true,	100,	12 );
} //end func

function Loot_ShowType(LootIndex) {
    if ((LootIndex==undefined) || (LootIndex==null)) LootIndex = 0;

    if (LootIndex==0) {
	var Begin = lootTypeBegin;
	var End   = lootTypeEnd;
    } else {
    	var Begin = LootIndex;
    	var End   = LootIndex;
    } // endif

    var Header = false;
    for (var x=Begin; x<=End; x++) {
    	// Show all loot types
	with (this.TypeInfo[x]) {
	    if (Enabled) {
		if (!Header) {
		    Header = true;
		    Chat.Screen(" ");
		    Chat.Screen(Util.Width("Type",1.2) + Util.Width("Enabled",1.0) +
		    	        Util.Width("MaxPickup",1.2) + "MinSlots");
		} // endif

		// Print info
		Chat.Screen(Util.Width(Name,1.2) + Util.Width(Enabled + " ",1.0) +
			    Util.Width(MaxCount + " ",1.2) + MinSlots);
	    } // endif
	} // end with
    } // endfor
    return Header;	// returns false if no items found, true otherwise
} // endfunc

// Check armor to see if it matches criteria.
function Loot_CheckArmor(ItemAco, ArmorIndex) {
    with (this.ArmorFilter[ArmorIndex]) {
	// Check AL level
	if (ItemAco.oai.iai.al < MinAL) return false;

	// Check workmanship
	if (ItemAco.workmanship > MaxWork) return false;

	// Check BU
	if (ItemAco.burden > MaxBU) return false;

	// Check value
	if (ItemAco.cpyValue > MaxValue) return false;

	// If minimum spell count is 0, then just return success since we just passed all the previous tests
	if (MinSpells == 0) return true;

	// Check spell count.  Note SpellCount is set in Loot_WhatType routine
	if (this.SpellCount < MinSpells) return false;

	// Check spell frequency.
	for (x=MinSplLvl; x<10; x++)
	if (this.SpellFreq[x] > 0) return true;
    } // end with

    // No spells found that meet minimum level requirement
    return false;
} // end function

function Loot_CheckWeapon(ItemAco, WeaponIndex) {
    if (LootDebug) Log.OutD("Loot_CheckWeapon: Begin, Item [" + ItemAco.szName + "] Index [" + WeaponIndex + "]");
    with (this.WeaponFilter[WeaponIndex]) {
	var AttBon = (Math.round(ItemAco.oai.iwi.scaleAttackBonus*1000)-1000)/10;
	var DefBon = (Math.round(ItemAco.oai.iwi.scaleDefenseBonus*1000)-1000)/10;
	var DamBon =  Math.round(ItemAco.oai.iwi.scaleDamageBonus*100)-100;
	if (LootDebug)
	    Log.OutD("Loot_CheckWeapon: AttBon [" + AttBon + "] DefBon [" + DefBon + "] DamBon [" + DamBon + "]");

	// Defense bonus too low?
	if (DefBon < MinDef) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed MinDef test");
	    return false;
	} // endif

	// For melee weapons, reject if attack bonus too low
	if ((ItemAco.eqm & eqmWeapon) && (AttBon < MinAtt)) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed MinAtt test");
	    return false;
	} // endif

	// Workmanship too high?
	if (ItemAco.workmanship > MaxWork) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed MaxWork test");
	    return false;
	} // endif

	// For melee weapons, reject if max damage too low
	if ((ItemAco.eqm & eqmWeapon) && (ItemAco.oai.iwi.dhealth < MinDam)) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed Melee MinDam test");
	    return false;
	} // endif

	// For ranged weapons, reject if damage bonus too low
	if ((ItemAco.eqm & eqmRangedWeapon) && (DamBon < MinDam)) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed Ranged MinDam test");
	    return false;
	} // endif

	// Reject if value to high
	if (ItemAco.cpyValue > MaxValue) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed max value test");
	    return false;
	} // endif

	// Reject if burden too high
	if (ItemAco.burden > MaxBU) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed max BU test");
	    return false;
	} // endif

	// If minimum spell count is 0, then just return success since we just passed all the previous tests
	if (MinSpells == 0) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: MinSpells=0, good weapon found.");
	    return true;
	} // endif

	// Check spell count.  Note SpellCount is set in Loot_WhatType routine
	if (this.SpellCount < MinSpells) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed min spells test");
	    return false;
	} // endif

	// Check spell frequency.  Note SpellCount is a global variable set in CheckItem routine
	for (x=MinSplLvl; x<10; x++) {
	    if (this.SpellFreq[x] > 0) {
	    	if (LootDebug) Log.OutD("Loot_CheckWeapon: Spl lvl test passed, good weapon found.");
		return true;
	    } // endif
	} // endfor
    } // end with

    // No spells found that meet minimum level requirement
    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed min spell lvl test");
    return false;
} // end function

function Loot_CheckMinSpellLvls(MinSplLvl, MinSplLvl2, MinSplLvl3) {
    var TempFreq = new Array();
    for (var x=0; x<10; x++) TempFreq[x] = this.SpellFreq[x];

    if (MinSplLvl>0) {
	for (var x=MinSplLvl; x<10; x++) {
	    if (TempFreq[x] > 0) {
		TempFreq[x]--;	// Decrement freq count for this level
		break;
	    } // endif
	} // endfor
	// If no spell matches mininum specified in 'MinSplLvl', then check failed.
	if (x==10) return false;
    } // endif

   if (MinSplLvl2>0) {
	for (var x=MinSplLvl2; x<10; x++) {
	    if (TempFreq[x] > 0) {
		TempFreq[x]--;	// Decrement freq count for this level
		break;
	    } // endif
	} // endfor
	// If no spell matches mininum specified in 'MinSplLvl2', then check failed.
	if (x==10) return false;
    } // endif

   if (MinSplLvl3>0) {
	for (var x=MinSplLvl3; x<10; x++) {
	    if (TempFreq[x] > 0) {
		TempFreq[x]--;	// Decrement freq count for this level
		break;
	    } // endif
	} // endfor
	// If no spell matches mininum specified in 'MinSplLvl3', then check failed.
	if (x==10) return false;
    } // endif

    // If we make it here than all minimum spell requirements have been met, return true
    return true;
} //end func

function Loot_WhatType(ItemAco) {
    if (LootDebug) Log.OutD("Loot_WhatType: Begin, Item [" + ItemAco.szName + "] Eqm [" + Util.Hex(ItemAco.eqm) + "]");

    var Material = ItemAco.material;
    var Burden   = ItemAco.burden;
    var Work     = ItemAco.workmanship;
    var Value    = ItemAco.cpyValue;

    if (LootDebug)
    	Log.OutD("Loot_WhatType: Material [" + Material + "] Bur [" + Burden + "] Wrk [" + Work + "] Val [" + Value + "]");

    // See if the item name matches any entries on 'lootItemList'
    if (LootDebug) Log.OutD("Loot_WhatType: Checking name match...");
    this.AlertFlag = false;
    for (var x=0; x<this.ItemFilter.length; x++) {
	if (this.ItemFilter[x].Name == ItemAco.szName) {
	    // Found match, see if we want to pick it up
	    if (this.ItemFilter[x].Alert) this.AlertFlag = true;
	    if (this.ItemFilter[x].Enabled)  {
		if (LootDebug) Log.OutD("Loot_WhatType: Match found");
		return lootTypeItem;
	    } // endif
	} // endif
    } // endfor
    if (LootDebug) Log.OutD("Loot_WhatType: Name match not found");

    // See if item is a 'rare'.  Note I have no idea what to look for at this point, so for now
    // we will just look for the word 'rare' in some of the description fields
    if (LootDebug) Log.OutD("Loot_WhatType: Checking RARE match...");
    if ((ItemAco.oai) && (ItemAco.oai.ibi)) {
    	var Text1 = ItemAco.oai.ibi.szDescSimple;
    	var Text2 = ItemAco.oai.ibi.szDescDetailed;
    	var Text3 = ItemAco.oai.ibi.szComment;
    	var Up1   = Text1.toUpperCase();
    	var Up2   = Text2.toUpperCase();
    	var Up3   = Text3.toUpperCase();

	if (Up1.match(/RARE/) || Up2.match(/RARE/) || Up3.match(/RARE/)) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Rare found.");
	    return lootTypeRare;
	} // endif
    } // endif

    // Blank out spell count and frequency array
    this.SpellCount = 0;	// # of spells found on item
    for (var x=0; x<10; x++) this.SpellFreq[x] = 0;

    // If this is equipment and an assess was successfully completed on this item, then process the spell
    // information on this item (if it has any).  While looking at the spell info, check for majors and
    // minors if desired.
    if ((ItemAco.eqm != 0) && (ItemAco.oai.iei)) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Generating spell information...");
	// Loop through all the spells on this item
	this.SpellCount = ItemAco.oai.iei.cspellid;
	for (var x=0; x<ItemAco.oai.iei.cspellid; x++) {
	    var Id       = ItemAco.oai.iei.rgspellid(x);
	    var SpellObj = skapi.SpellInfoFromSpellid(Id);
	    var Name     = SpellObj.szName;

	    // Compute spell level from difficulty and update spell frequency array
	    var Lvl = Math.round(SpellObj.diff/50)+1;
	    this.SpellFreq[Lvl]++;
	    if (LootDebug) Log.OutD("Spell # [" + x + "] Id [" + Id + "] Lvl [" + Lvl + "] Name [" + Name + "]");

	    // Check for majors if desired
	    if (this.TypeInfo[lootTypeMajor].Enabled) {
	    	if (this.MajorIndex[Id] != undefined) {
		    var Offset = this.MajorIndex[Id];
		    if (this.MajorFilter[Offset].Enabled) {
			if (LootDebug) Log.OutD("Loot_WhatType: Major [" + Name + "] found");
			return lootTypeMajor;
		    } // endif
		} // endif
	    } // endif

	    // Check for minors if desired
	    if (this.TypeInfo[lootTypeMinor].Enabled) {
	    	if (this.MinorIndex[Id] != undefined) {
		    var Offset = this.MinorIndex[Id];
		    if (this.MinorFilter[Offset].Enabled && (this.MinorFilter[Offset].Eqm & ItemAco.eqm)) {
			if (LootDebug) Log.OutD("Loot_WhatType: Minor [" + Name + "] found");
			return lootTypeMinor;
		    } // endif
		} // endif
	    } // endif
	} // endfor
    } // endif

    // If this item is underclothes and flag is set to pick them up, then see if we meet the
    // minimum spell requirements.   If it's a boot (which has LowerLegsUnder set) then reject.
    if (this.TypeInfo[lootTypeClothing].Enabled && !(ItemAco.eqm & eqmFeet) &&
        (ItemAco.eqm & (eqmChestUnder | eqmAbdomenUnder | eqmUpperArmsUnder |
			eqmUpperLegsUnder | eqmLowerLegsUnder))) {
	if (this.CheckMinSpellLvls(this.ClothingFilter.MinSplLvl1, this.ClothingFilter.MinSplLvl2,
				   this.ClothingFilter.MinSplLvl3)) {
	    // Good clothing found
	    if (LootDebug) Log.OutD("Loot_WhatType: Good clothing found");
	    return lootTypeClothing;
	} // endif
    } // endif

    // If we want to pickup armor, since item is armor, and armor assess information exists then check it
    if (this.TypeInfo[lootTypeArmor].Enabled && (ItemAco.eqm & armorAll) && (ItemAco.oai.iai)) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Armor found, examining...");
    	// See if item is covenant armor
    	if (ItemAco.szName.match(/Covenant/))
	    var CovArmor = true;
	else
	    var CovArmor = false;

	// Cycle through the armor filter entries looking for a match
	for (var x=0; x<this.ArmorFilter.length; x++)
	    if (this.ArmorFilter[x].Enabled &&
	        (this.ArmorFilter[x].Eqm & ItemAco.eqm) &&
	        (this.ArmorFilter[x].CovFlag == CovArmor)) break;

	// If match found, check armor item against criteria
	if (x < this.ArmorFilter.length) {
	    if (this.CheckArmor(ItemAco, x)) {
	    	// Good armor found
		if (LootDebug) Log.OutD("Loot_WhatType: Good armor found");
		return lootTypeArmor;
	    } // endif
	} //endif
    } //endif

    // If we want to pickup weapons, and this item is a weapon and we have assess weapon info then check it
    if (this.TypeInfo[lootTypeWeapon].Enabled && (ItemAco.eqm & (eqmWeapon|eqmRangedWeapon)) && (ItemAco.oai.iwi)) {
    	if (LootDebug) {
	    Log.OutD("Loot_WhatType: Weapon found, determining type, filter len [" + this.WeaponFilter.length + "]");
	    Log.OutD("Loot_WhatType: Weap skid [" + ItemAco.oai.iwi.skid + "] Weap dmty [" + ItemAco.oai.iwi.dmty + "]");
	} // endif

	// Cycle through the weapon filter entries looking for a match
	for (var x=0; x<this.WeaponFilter.length; x++) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Index [" + x + "] Name [" + this.WeaponFilter[x].Name +
	    			    "] Skid [" + this.WeaponFilter[x].Skid +
	    			    "] DamTypes [" + this.WeaponFilter[x].DamTypes+ "]");
	    if (this.WeaponFilter[x].Enabled &&
	        (this.WeaponFilter[x].Skid == ItemAco.oai.iwi.skid) &&
		(this.WeaponFilter[x].DamTypes == ItemAco.oai.iwi.dmty)) break;
	} // endfor

	// If match found, check armor item against criteria
	if (x < this.WeaponFilter.length) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Weapon type [" + this.WeaponFilter[x].Name + "] found.");
	    if (this.CheckWeapon(ItemAco, x)) {
	    	// Good weapon found
		if (LootDebug) Log.OutD("Loot_WhatType: Good weapon found.");
		return lootTypeWeapon;
	    } // endif
	} else {
	    if (LootDebug) Log.OutD("Loot_WhatType: Unknown weapon type.");
	} //endif
    } //endif

    // If we want to pickup nice jewelry and this item is equipable in necklace,
    // bracelet, or ring slots, then check it out
    if (this.TypeInfo[lootTypeJewelry].Enabled && (ItemAco.eqm & (eqmNecklace|eqmBracelets|eqmRings))) {
	// See if at least one spell at the required level...
	if (this.CheckMinSpellLvls(this.JewelryFilter.MinSplLvl1, this.JewelryFilter.MinSplLvl2,
				   this.JewelryFilter.MinSplLvl3)) {
	    // Spell found at or above the required level, we have a winner
	    if (LootDebug) Log.OutD("Loot_WhatType: Good jewelry found");
	    return lootTypeJewelry;
	} // endif
    } // endif

    if (this.TypeInfo[lootTypeWand].Enabled && (ItemAco.eqm & eqmFocusWeapon) && (ItemAco.oai.ibi)) {
	var ManaConvMod = ItemAco.oai.ibi.fractManaConvMod;
	var MeleeDMod   = (Math.round(ItemAco.oai.iwi.scaleDefenseBonus*1000)-1000)/10;
	var MatchMana	= (ManaConvMod >= this.WandFilter[Work].MinManaC/100);
	var MatchMeleeD = (MeleeDMod >= this.WandFilter[Work].MinMeleeD);
	if (LootDebug) Log.OutD("Loot_WhatType: Checking wand, mana conv mod  [" + ManaConvMod +
				"] meleeD mod [" + MeleeDMod + "]");
	if ((this.WandFilter[Work].Enabled) && (Value <= this.WandFilter[Work].MaxValue) &&
	    (((this.WandFilter[Work].AndOrFlag == 0) && MatchMana && MatchMeleeD) ||
	     ((this.WandFilter[Work].AndOrFlag == 1) && (MatchMana || MatchMeleeD))) ) {
	    // Found nice wand
	    if (LootDebug) Log.OutD("Loot_WhatType: Nice wand found");
	    return lootTypeWand;
	} else {
	    if (LootDebug) Log.OutD("Loot_WhatType: Wand value too high or ManaC mod too low for work level");
	} // endif
    } // endif

    // If the item is a scroll, see if it matches are criteria for scroll pickup.
    if (this.TypeInfo[lootTypeScroll].Enabled && (ItemAco.mcm & mcmBooksPaper) &&
        (ItemAco.szName.match(/Scroll/))) {
	// See if scroll is within level restrictions and if we only want scrolls that are in our spell book
	var SpellId     = ItemAco.spellid;
	var SpellObj    = skapi.SpellInfoFromSpellid(SpellId);
	var SpellLvl    = Math.floor(SpellObj.diff/50)+1;
	var InSpellBook = skapi.FSpellidInSpellbook(SpellId);
	var OnLootList  = FindScrollOnLootList(ItemAco.szName);
	if (LootDebug) {
	    Log.OutD("Checking scroll [" + ItemAco.szName + "] diff [" + SpellObj.diff + "] Lvl [" +
		     SpellLvl + "] Inbook [" + InSpellBook + "] OnLootList [" + OnLootList + "]");
	    Log.OutD("FilterMin [" +  this.ScrollFilter.MinLvl + "] FilterMax [" + this.ScrollFilter.MaxLvl + "]");
	    Log.OutD("UnknownOnly [" + this.ScrollFilter.UnknownOnly + "]");
	} // endif
	if ((SpellLvl >= this.ScrollFilter.MinLvl) &&
	    (SpellLvl <= this.ScrollFilter.MaxLvl) &&
	    (!this.ScrollFilter.UnknownOnly || (this.ScrollFilter.UnknownOnly && !InSpellBook && !OnLootList))) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Scroll found");
	    return lootTypeScroll;
	} // endif
    } // endif

    // If we want to look for tink salvage, see if this items workmanship is high enough for it's
    // material type and its burden is not too high.  Don't do burden check if autoUST enabled
    if (this.TypeInfo[lootTypeSalvage].Enabled) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Checking for tink salvage");
	// See if this material is defined on our salvage list
	if ((this.SalvageIndex[Material] == undefined) || (this.SalvageIndex[Material] == null)) {
	    if (LootDebug) Log.OutD("Loot_WhatType: No filter record for this material, skipping tink salvage check.");
	} else {
	    // Material defined, see if item's workmanship is high enough and BU not too high.
	    // Don't include bags of salvage in this category
	    var Offset = this.SalvageIndex[Material];
	    if (LootDebug) Log.OutD("Loot_WhatType: MinWork [" + this.SalvageFilter[Offset].MinWork + "]");
	    if ((this.SalvageFilter[Offset].Enabled) &&
	        (Work >= this.SalvageFilter[Offset].MinWork) &&
	        !(ItemAco.mcm & mcmSalvageBag)) {
	    	// Found a good salvage item, see if light enough and space in pack.  If we
	    	// are auto-USTing then ignore burden since item will be destroyed shortly
	    	if (!flagAutoUst && (Burden > this.SalvageFilter[Offset].MaxBU)) {
    	    	    if (LootDebug) Log.OutD("Loot_WhatType: Good tink salvage found but too heavy");
    	    	} else {
		    if (LootDebug) Log.OutD("Loot_WhatType: Good tink salvage found");
		    return lootTypeSalvage;
		} // endif
	    } // endif
	} // endif
	if (LootDebug) Log.OutD("Loot_WhatType: Not tink salvagable.");
    } // endif

    // If we want to look for valuable salvage, see if this items price to workmanship ratio is high enough
    // Note that USTing must be enabled before this category of loot will be picked up.
    if (this.TypeInfo[lootTypeValSalv].Enabled && flagAutoUst) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Checking for valuable salvage");
	// See if this material is defined on our salvage list
	if ((this.SalvageIndex[Material] == undefined) || (this.SalvageIndex[Material] == null)) {
	    if (LootDebug) Log.OutD("Loot_WhatType: No filter record for this material, skipping val salvage check.");
	} else {
	    // Material defined, see if item's price to workmanship ratio is high enough.
	    // Don't include bags of salvage in this category or materials that are being picked
	    // up as tink salvage.
	    var Offset = this.SalvageIndex[Material];
	    if (LootDebug) Log.OutD("Loot_WhatType: Ratio [" + this.ValSalvFilter.MinRatio + "]");
	    if (!this.SalvageFilter[Offset].ExcludeVal && (Work>0) && 
	       ((Value/Work) >= this.ValSalvFilter.MinRatio) && !(ItemAco.mcm & mcmSalvageBag)) {
	    	// Found a good valuable salvage item
		if (LootDebug) Log.OutD("Loot_WhatType: Good valuable salvage found");
		return lootTypeValSalv;
	    } // endif
	} // endif
	if (LootDebug) Log.OutD("Loot_WhatType: Not val salvagable.");
    } // endif

    // Item failed all previous tests.  For last test we see if it is a valuable item
    if (this.TypeInfo[lootTypeValuable].Enabled) {
	if (LootDebug) Log.OutD("Loot_WhatType: Checking value...");
	// See if item above value minimum, below burden maximum, and is not a bag of salvage or pyreals
	if ((Value >= this.ValuableFilter.MinValue) && (Burden <= this.ValuableFilter.MaxBU) &&
	    !(ItemAco.mcm & (mcmSalvageBag|mcmPyreal))) {
	    // Found valuable loot
	    if (LootDebug) Log.OutD("Loot_WhatType: Expensive loot found");
	    return lootTypeValuable;
	} else {
	    if (LootDebug) Log.OutD("Loot_WhatType: Not valuable or too heavy");
	} // endif
    } // endif

    // If we make it here, then item is worthless to us.  Return junk item type.
    return lootTypeJunk;
} // end function

// LOOT METHODS ***********************************************************

// Look for an OID on a loot list
function Loot_LookupLoot(Oid, Type) {
    if ((Type==undefined) || (Type==null)) Type = -1;

    // If type specified, then only search that type, else search all lists.
    if (Type < 0) {
	Begin = 0;
	End = lootTypeMax;
    } else {
    	Begin = Type;
    	End = Type;
    } // endif

    for (var x=Begin; x<=End; x++) {
    	with (this.TypeInfo[x]) {
	    for (var y=0; y<ItemCount; y++) {
		if (Oid == ItemList[y].Oid) {
		    this.TypeIndex = x;
		    this.ItemIndex = y;
		    return true;
		} // endif
	    } // end for
	} // end with
    } // end for
    return false;
} // end func

// Add an item to a loot list
function Loot_AddLoot(Aco, Type, Desc) {
    if (LootDebug) Log.OutD("Loot_AddLoot: Begin, Name [" + Aco.szName + "] Type [" + Type +
		 	    "] Desc [" + Desc + "]", LootDebugScreen);

    if (this.LookupLoot(Aco.oid, Type)) {
    	if (LootDebug) Log.OutD("Loot_AddLoot: Duplicate", LootDebugScreen);
    	return false; 	// Item already on list
    } // endif

    with (this.TypeInfo[Type]) {
    	ItemList[ItemCount] = new LootItemRec(Aco, Desc);
    	ItemCount++;
    	TotalValue += Aco.cpyValue;
    } // end with
    return true;
} // end func

// Remove an item from a loot list
function Loot_RemoveLoot(Oid) {
    if (!(this.LookupLoot(Oid, Type))) return false;	// Item not found

    // Decrement list count and move the last item to the deleted items
    // spot unless it was at the end of the list
    with (this.TypeInfo[this.TypeIndex]) {
	ItemCount--;
	if (this.ItemIndex < ItemCount) ItemList[this.ItemIndex] = ItemList[ItemCount];
    } // end with
    return true;
} // end func

// Refresh loot lists due to a logoff/login
function Loot_RefreshLoot() {
    for (var x=0; x<lootTypeTotal; x++) {
    	with (this.TypeInfo[x]) {
	    var y=0;
	    while (y<ItemCount) {
	    	NewAco = skapi.AcoFromOid(ItemList[y].Oid);
	    	if (NewAco && NewAco.fInInventory) {
		    ItemList[y].Aco = NewAco;
		    y++;
		} else {
		    ItemCount--;
		    if (y<ItemCount) ItemList[y] = ItemList[ItemCount];
		} // endif
	    } // end while
	} // end with
    } // end for
} // end func

function Loot_ResetLoot() {
  // First clear out loot counters
    for (var x=0; x<lootTypeTotal; x++) {
	this.TypeInfo[x].ItemCount = 0;
	this.TypeInfo[x].TotalValue = 0;
    } // endfor
} // end func

// ITEM METHODS ***********************************************************

function Loot_EnableItem(Index) {
    if (LootDebug) Log.OutD("Loot_EnableItem: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ItemFilter[Index].Enabled = true;
    if (LootDebug) Log.OutD("Loot_EnableItem: End", LootDebugScreen);
} // end func

function Loot_DisableItem(Index) {
    if (LootDebug) Log.OutD("Loot_DisableItem: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ItemFilter[Index].Enabled = false;
    if (LootDebug) Log.OutD("Loot_DisableItem: End", LootDebugScreen);
} // end func

function Loot_SetItem(Name, Enabled, Alert) {
    if (LootDebug) Log.OutD("Loot_SetItem: Begin, Name [" + Name + "]", LootDebugScreen);
    var Index = this.LookupItem(Name);
    if (Index < 0) return false;
    this.ItemFilter[Index].Enabled = Enabled;
    this.ItemFilter[Index].Alert   = Alert;
    if (LootDebug) Log.OutD("Loot_SetItem: End", LootDebugScreen);
    return true;
} // end func

function Loot_ModItem(Index, Alert) {
    if (LootDebug) Log.OutD("Loot_ModItem: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ItemFilter[Index].Alert = Alert;
    if (LootDebug) Log.OutD("Loot_ModItem: End", LootDebugScreen);
    return true;
} // end func

function Loot_LookupItem(Name) {
    if (LootDebug2) Log.OutD("Loot_LookupItem: Begin, Name [" + Name + "]", LootDebugScreen);
    for (var x=0; x<this.ItemFilter.length; x++) if (this.ItemFilter[x].Name == Name) {
    	if (LootDebug) Log.OutD("Loot_LookupItem: End, Found [" + x + "]", LootDebugScreen);
    	return x;
    } // endfor
    if (LootDebug2) Log.OutD("Loot_LookupItem: End, not found", LootDebugScreen);
    return -1;	  // not found
} // end func

function Loot_ShowItem(EnableOnly) {
    if ((EnableOnly==undefined) || (EnableOnly==null)) EnableOnly = false;
    if (LootDebug) Log.OutD("Loot_Showitem: Begin, EnableOnly [" + EnableOnly + "]", LootDebugScreen);

    var Header = false;
    for (var x=0; x<this.ItemFilter.length; x++) {
	with (this.ItemFilter[x]) {
	    if (EnableOnly && !Enabled && !Alert) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width("Item",4.0) + Util.Width("Enabled",1.0) + "Alert");
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,4.0) + Util.Width(Enabled + " ",1.0) + Alert);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowItem: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetItem() {
    if (LootDebug) Log.OutD("Loot_ResetItem: Begin", LootDebugScreen);
    for (var x=0; x<this.ItemFilter.length; x++) {
	this.ItemFilter[x].Enabled = false;
	this.ItemFilter[x].Alert   = false;
    } // end for
    if (LootDebug) Log.OutD("Loot_ResetItem: End", LootDebugScreen);
} // end func

// MAJOR METHODS ***********************************************************

function Loot_EnableMajor(Index) {
    if (LootDebug) Log.OutD("Loot_EnableMajor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MajorFilter[Index] = true;
    if (LootDebug) Log.OutD("Loot_EnableMajor: End", LootDebugScreen);
} // end func

function Loot_DisableMajor(Index) {
    if (LootDebug) Log.OutD("Loot_DisableMajor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MajorFilter[Index] = false;
    if (LootDebug) Log.OutD("Loot_DisableMajor: End", LootDebugScreen);
} // end func

function Loot_SetMajor(Name, Enabled) {
    if (LootDebug) Log.OutD("Loot_SetMajor: Begin, Name [" + Name + "]", LootDebugScreen);
    var SpellId = skapi.SpellidFromSz(Name);
    if (this.MajorIndex[SpellId] == undefined) return false;
    var Index = this.MajorIndex[SpellId];
    this.MajorFilter[Index].Enabled = Enabled;
    if (LootDebug) Log.OutD("Loot_MajorItem: End", LootDebugScreen);
    return true;
} // end func

function Loot_ShowMajor(EnableOnly) {
    if ((EnableOnly==undefined) || (EnableOnly==null)) EnableOnly = false;
    if (LootDebug) Log.OutD("Loot_ShowMajor: Begin, EnableOnly [" + EnableOnly + "]", LootDebugScreen);

    var Header = false;
    for (var x=0; x<this.MajorFilter.length; x++) {
    	// Show major type
	with (this.MajorFilter[x]) {
	    if (EnableOnly && !Enabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width("Major",3.5) + Util.Width("Enabled",1.0));
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,3.5) + Util.Width(Enabled + " ",1.0));
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowMajor: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetMajor() {
    if (LootDebug) Log.OutD("Loot_ResetMajor: Begin", LootDebugScreen);
    for (var x=0; x<this.MajorFilter.length; x++) this.MajorFilter[x].Enabled = true;
    if (LootDebug) Log.OutD("Loot_ResetMajor: End", LootDebugScreen);
} // end func


// MINOR METHODS ***********************************************************

function Loot_EnableMinor(Index) {
    if (LootDebug) Log.OutD("Loot_EnableMinor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MinorFilter[Index] = true;
    if (LootDebug) Log.OutD("Loot_EnableMinor: End", LootDebugScreen);
} // end func

function Loot_DisableMinor(Index) {
    if (LootDebug) Log.OutD("Loot_DisableMinor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MinorFilter[Index] = false;
    if (LootDebug) Log.OutD("Loot_DisableMinor: End", LootDebugScreen);
} // end func

function Loot_SetMinor(Name, Enabled, Eqm) {
    if (LootDebug) Log.OutD("Loot_SetMinor: Begin, Name [" + Name + "]", LootDebugScreen);
    var SpellId = skapi.SpellidFromSz(Name);
    if (this.MinorIndex[SpellId] == undefined) return false;
    var Index = this.MinorIndex[SpellId];
    this.MinorFilter[Index].Enabled = Enabled;
    this.MinorFilter[Index].Eqm	    = Eqm;
    if (LootDebug) Log.OutD("Loot_SetMinor: End", LootDebugScreen);
    return true;
} // end func

function Loot_ModMinor(Index, Eqm) {
    if (LootDebug) Log.OutD("Loot_ModMinor: Begin, Index [" + Index + "]", LootDebugScreen);
    if ((Eqm==undefined) || (Eqm==null)) Eqm = -1;
    if (Eqm >= 0) this.MinorFilter[Index].Eqm = Eqm;
    if (LootDebug) Log.OutD("Loot_ModMinor: End", LootDebugScreen);
} // end func

function Loot_ShowMinor(Index) {
    if ((Index==undefined) || (Index==null)) Index = -1;
    if (LootDebug) Log.OutD("Loot_ShowMajor: Begin, Index [" + Index + "]", LootDebugScreen);

    if (Index<0) {
	var Begin = 0;
	var End   = this.MinorFilter.length-1;
    } else {
    	var Begin = Index;
    	var End   = Index;
    } // endif

    var Header = false;
    for (var x=Begin; x<=End; x++) {
    	// Show major type
	with (this.MinorFilter[x]) {
	    if (Enabled) {
		if (!Header) {
		    Header = true;
		    Chat.Screen(" ");
		    Chat.Screen(Util.Width("Minor",3.5) + Util.Width("Enabled",1.0) + "Eqm");
		} // endif

		// Print info. First convert eqm to test
		if (Eqm) {
		    EqmText = "";
		    if (Eqm & eqmHead)		  EqmText = EqmText + "hd  ";
		    if (Eqm & eqmHands)		  EqmText = EqmText + "ha  ";
		    if (Eqm & eqmFeet)		  EqmText = EqmText + "ft  ";
		    if (Eqm & eqmChestOuter)	  EqmText = EqmText + "ch  ";
		    if (Eqm & eqmUpperArmsOuter)  EqmText = EqmText + "ua  ";
		    if (Eqm & eqmLowerArmsOuter)  EqmText = EqmText + "la  ";
		    if (Eqm & eqmAbdomenOuter)	  EqmText = EqmText + "ab  ";
		    if (Eqm & eqmUpperLegsOuter)  EqmText = EqmText + "ul  ";
		    if (Eqm & eqmLowerLegsOuter)  EqmText = EqmText + "ll  ";
		} else {
		    EqmText = "None";
		} // endif

		Chat.Screen(Util.Width(Name,3.5) + Util.Width(Enabled + " ",1.0) + EqmText);
	    } // endif
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowMinor: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetMinor() {
    if (LootDebug) Log.OutD("Loot_ResetMinor: Begin", LootDebugScreen);

    for (var x=0; x<this.MinorFilter.length; x++) {
    	this.MinorFilter[x].Enabled = false;
    	this.MinorFilter[x].Eqm     = 0;
    } // endfor

    // Magic skills
    this.SetMinor("Minor Creature Enchantment Aptitude", true,	armorExtremity	);
    // this.SetMinor("Minor Item Enchantment Aptitude",	 false,	0		);
    this.SetMinor("Minor Life Magic Aptitude",		 true,	armorExtremity	);
    this.SetMinor("Minor War Magic Aptitude",		 true,	armorExtremity	);
    // this.SetMinor("Minor Magic Resistance",		 false,	0		);
    // this.SetMinor("Minor Mana Conversion Prowess",	 false,	0		);

    // Melee skills
    this.SetMinor("Minor Axe Aptitude",		false,	armorExtremity	);
    this.SetMinor("Minor Bow Aptitude",		true,	armorExtremity	);
    this.SetMinor("Minor Crossbow Aptitude",	true,	armorExtremity	);
    // this.SetMinor("Minor Dagger Aptitude",	false,	0		);
    // this.SetMinor("Minor Mace Aptitude",	false,	0		);
    // this.SetMinor("Minor Spear Aptitude",	false,	0		);
    // this.SetMinor("Minor Staff Aptitude",	false,	0		);
    this.SetMinor("Minor Sword Aptitude",	true,	armorExtremity	);
    // this.SetMinor("Minor Thrown Aptitude",	false,	0		);
    this.SetMinor("Minor Unarmed Aptitude",	true,	armorExtremity	);
    // this.SetMinor("Minor Impregnability",	false,	0		);
    // this.SetMinor("Minor Invulnerability",	false,	0		);

    // this.SetMinor("Minor Armor Tinkering Expertise",	  false,	0		);
    // this.SetMinor("Minor Item Tinkering Expertise",	  false,	0		);
    // this.SetMinor("Minor Magic Item Tinkering Expertise", false,	0		);
    // this.SetMinor("Minor Weapon Tinkering Expertise",  false,	0		);
    // this.SetMinor("Minor Alchemical Prowess",	  false,	0		);
    // this.SetMinor("Minor Cooking Prowess",		  false,	0		);
    // this.SetMinor("Minor Fletching Prowess",		  false,	0		);

    // Miscellaneous
    // this.SetMinor("Minor Arcane Prowess",	false,	0	);
    // this.SetMinor("Minor Deception Prowess",	false,	0	);
    // this.SetMinor("Minor Fealty",		false,	0	);
    // this.SetMinor("Minor Healing Prowess",	false,	0	);
    // this.SetMinor("Minor Jumping Prowess",	false,	0	);
    // this.SetMinor("Minor Leadership",	false,	0	);
    // this.SetMinor("Minor Lockpick Prowess",	false,	0	);
    // this.SetMinor("Minor Monster Attunement",false,	0	);
    // this.SetMinor("Minor Person Attunement",	false,	0	);
    // this.SetMinor("Minor Sprint",		false,	0	);

    // Weapon modifiers.  These don't buy much by themselves.  A good weapon will be
    // picked up regardless, so disable these by default.
    // this.SetMinor("Minor Blood Thirst",	false,	0	);
    // this.SetMinor("Minor Defender",		false,	0	);
    // this.SetMinor("Minor Heart Thirst",	false,	0	);
    // this.SetMinor("Minor Swift Hunter",	false,	0	);

    // Wards and banes.  Mostly useless
    // this.SetMinor("Minor Armor", 		false,	0	);
    // this.SetMinor("Minor Acid Ward",		false,	0	);
    // this.SetMinor("Minor Bludgeoning Ward",	false,	0	);
    // this.SetMinor("Minor Flame Ward",	false,	0	);
    // this.SetMinor("Minor Frost Ward",	false,	0	);
    // this.SetMinor("Minor Piercing Ward",	false,	0	);
    // this.SetMinor("Minor Slashing Ward",	false,	0	);
    // this.SetMinor("Minor Storm Ward",	false,	0	);

    // this.SetMinor("Minor Impenetrability",	false,	0	);
    // this.SetMinor("Minor Acid Bane",		false,	0	);
    // this.SetMinor("Minor Bludgeoning Bane",	false,	0	);
    // this.SetMinor("Minor Flame Bane",	false,	0	);
    // this.SetMinor("Minor Frost Bane",	false,	0	);
    // this.SetMinor("Minor Piercing Bane",	false,	0	);
    // this.SetMinor("Minor Slashing Bane",	false,	0	);
    // this.SetMinor("Minor Storm Bane",	false,	0	);

    if (LootDebug) Log.OutD("Loot_ResetMinor: End", LootDebugScreen);
} // end func


// ARMOR METHODS ***********************************************************

function Loot_EnableArmor(Index) {
    if (LootDebug) Log.OutD("Loot_EnableArmor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ArmorFilter[Index].Enabled = true;
    if (LootDebug) Log.OutD("Loot_EnableArmor: End", LootDebugScreen);
} // end func

function Loot_DisableArmor(Index) {
    if (LootDebug) Log.OutD("Loot_DisableArmor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ArmorFilter[Index].Enabled = false;
    if (LootDebug) Log.OutD("Loot_DisableArmor: End", LootDebugScreen);
} // end func

function Loot_AddArmor(Name, Eqm, CovFlag) {
    if (LootDebug) Log.OutD("Loot_AddArmor: Begin, Name [" + Name + "]", LootDebugScreen);
    if (this.LookupArmor(Name) >= 0) return false;	// Item already exists
    var NextSlot = this.ArmorFilter.length;
    this.ArmorFilter[NextSlot] = new ArmorFilterRec(Name, Eqm, CovFlag);
    this.ArmorOpts = this.ArmorOpts + " <option data='" + NextSlot + "' text='" + Name + "'/>\n";
    if (LootDebug) Log.OutD("Loot_AddArmor: End", LootDebugScreen);
    return true;
} // end func

function Loot_SetArmor(Name, Enabled, MinAL, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl) {
    if (LootDebug) Log.OutD("Loot_SetArmor: Begin, Name [" + Name + "]", LootDebugScreen);
    var Index = this.LookupArmor(Name);
    if (Index < 0) return false;
    this.ArmorFilter[Index].Enabled	= Enabled;
    this.ArmorFilter[Index].MinAL	= MinAL;
    this.ArmorFilter[Index].MaxWork	= MaxWork;
    this.ArmorFilter[Index].MaxBU	= MaxBU;
    this.ArmorFilter[Index].MaxValue	= MaxValue;
    this.ArmorFilter[Index].MinSpells	= MinSpells;
    this.ArmorFilter[Index].MinSplLvl	= MinSplLvl;
    if (LootDebug) Log.OutD("Loot_SetArmor: End", LootDebugScreen);
    return true;
} // end func

function Loot_ModArmor(Index, MinAL, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl) {
    if (LootDebug) Log.OutD("Loot_ModArmor: Begin, Index [" + Index + "]", LootDebugScreen);
    if ((MinAL     == undefined) || (MinAL     == null)) MinAL     = -1;
    if ((MaxWork   == undefined) || (MaxWork   == null)) MaxWork   = -1;
    if ((MaxBU     == undefined) || (MaxBU     == null)) MaxBU     = -1;
    if ((MaxValue  == undefined) || (MaxValue  == null)) MaxValue  = -1;
    if ((MinSpells == undefined) || (MinSpells == null)) MinSpells = -1;
    if ((MinSplLvl == undefined) || (MinSplLvl == null)) MinSplLvl = -1;

    if (MinAL     >= 0) this.ArmorFilter[Index].MinAL	   = MinAL;
    if (MaxWork   >= 0) this.ArmorFilter[Index].MaxWork	   = MaxWork;
    if (MaxBU     >= 0) this.ArmorFilter[Index].MaxBU	   = MaxBU;
    if (MaxValue  >= 0) this.ArmorFilter[Index].MaxValue   = MaxValue;
    if (MinSpells >= 0) this.ArmorFilter[Index].MinSpells  = MinSpells;
    if (MinSplLvl >= 0) this.ArmorFilter[Index].MinSplLvl  = MinSplLvl;
    if (LootDebug) Log.OutD("Loot_ModArmor: End", LootDebugScreen);
    return true;
} // end func

function Loot_ShowArmor(Index) {
    if ((Index==undefined) || (Index==null)) Index = -1;
    if (LootDebug) Log.OutD("Loot_ShowArmor: Begin, Index [" + Index + "]", LootDebugScreen);

    if (Index<0) {
	var Begin = 0;
	var End   = this.ArmorFilter.length-1;
    } else {
    	var Begin = Index;
    	var End   = Index;
    } // endif

    var Header = false;
    for (var x=Begin; x<=End; x++) {
	with (this.ArmorFilter[x]) {
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width(" ",3.2) + Util.Width("Min",0.8) + Util.Width("Max",0.8) +
			    Util.Width("Max",0.8) +  Util.Width("Max",0.8) + Util.Width("Min",0.8) + "MinSpl");
		Chat.Screen(Util.Width("Armor",2.0) + Util.Width("Enabled",1.2) + Util.Width("AL",0.8) +
			    Util.Width("BU",0.8) + Util.Width("Val",0.8) + Util.Width("Work",0.8) +
			    Util.Width("Spl",0.8) + "Lvl");
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,2.2) + Util.Width(Enabled+" ",1.0) + Util.Width(MinAL+" ",0.8) +
			Util.Width(MaxBU+" ",0.8) + Util.Width(MaxValue+ " ",0.8) + Util.Width(MaxWork+" ",0.8) +
			Util.Width(MinSpells+" ",0.8) + MinSplLvl);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowArmor: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetArmor() {
    if (LootDebug) Log.OutD("Loot_ResetArmor: Begin", LootDebugScreen);
    // SetArmor(  Name,               Enabled, MinAL, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl)
    this.SetArmor("Armor (Body)",	true,	225,	 5,	4000,	40000,	0,	6);
    this.SetArmor("Armor (Head)",	true,	295,	 5,	1000,	20000,	0,	6);
    this.SetArmor("Armor (Hands)",	true,	225,	 5,	1000,	20000,	0,	6);
    this.SetArmor("Armor (Feet)",	true,	225,	 5,	1000,	20000,	0,	6);
    this.SetArmor("Armor (Shield)",	true,	160,	 5,	2000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Body)",	true,	350,	10,	4000,	40000,	0,	6);
    this.SetArmor("Cov Armor (Head)",	true,	350,	10,	1000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Hands)",	true,	350,	10,	1000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Feet)",	true,	350,	10,	1000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Shield)",	true,	350,	10,	2000,	20000,	0,	6);
    if (LootDebug) Log.OutD("Loot_ResetArmor: End", LootDebugScreen);
} // end func

function Loot_LookupArmor(Name) {
    if (LootDebug2) Log.OutD("Loot_LookupArmor: Begin, Name [" + Name + "]", LootDebugScreen);
    for (var x=0; x<this.ArmorFilter.length; x++) if (this.ArmorFilter[x].Name == Name) {
   	if (LootDebug) Log.OutD("Loot_LookupArmor: End, Found [" + x + "]", LootDebugScreen);
    	return x;
    } // endfor
    if (LootDebug2) Log.OutD("Loot_LookupArmor: End, not found", LootDebugScreen);
    return -1;   // not found
} // end func


// WEAPON METHODS ***********************************************************
function Loot_EnableWeapon(Index) {
    if (LootDebug) Log.OutD("Loot_EnableWeapon: Begin, Index [" + Index + "]", LootDebugScreen);
    this.WeaponFilter[Index].Enabled = true;
    if (LootDebug) Log.OutD("Loot_EnableWeapon: End", LootDebugScreen);
} // end func

function Loot_DisableWeapon(Index) {
    if (LootDebug) Log.OutD("Loot_DisableWeapon: Begin, Index [" + Index + "]", LootDebugScreen);
    this.WeaponFilter[Index].Enabled = false;
    if (LootDebug) Log.OutD("Loot_DisableWeapon: End", LootDebugScreen);
} // end func

function Loot_AddWeapon(Name, Skid, DamTypes) {
    if (LootDebug) Log.OutD("Loot_AddWeapon: Begin, Name [" + Name + "]", LootDebugScreen);
    if (this.LookupWeapon(Name) >= 0) return false;	// Item already exists
    var NextSlot = this.WeaponFilter.length;
    this.WeaponFilter[NextSlot] = new WeaponFilterRec(Name, Skid, DamTypes);
    this.WeaponOpts = this.WeaponOpts + " <option data='" + NextSlot + "' text='" + Name + "'/>\n";
    if (LootDebug) Log.OutD("Loot_AddWeapon: End", LootDebugScreen);
    return true;
} // end func

function Loot_SetWeapon(Name, Enabled, MinDam, MinAtt, MinDef, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl) {
    if (LootDebug) Log.OutD("Loot_SetWeapon: Begin, Name [" + Name + "] Enabled [" + Enabled +
    			    "] MinDam [" + MinDam + "]", LootDebugScreen);
    var Index = this.LookupWeapon(Name);
    if (Index < 0) {
    	Chat.Screen("%Config error, unknown weapon [" + Name + "]");
	return false;
    } // endif
    this.WeaponFilter[Index].Enabled	= Enabled;
    this.WeaponFilter[Index].MinDam	= MinDam;
    this.WeaponFilter[Index].MinAtt	= MinAtt;
    this.WeaponFilter[Index].MinDef	= MinDef;
    this.WeaponFilter[Index].MaxWork	= MaxWork;
    this.WeaponFilter[Index].MaxBU	= MaxBU;
    this.WeaponFilter[Index].MaxValue	= MaxValue;
    this.WeaponFilter[Index].MinSpells	= MinSpells;
    this.WeaponFilter[Index].MinSplLvl	= MinSplLvl;
    if (LootDebug) Log.OutD("Loot_SetWeapon: End", LootDebugScreen);
    return true;
} // endif

function Loot_ModWeapon(Index, MinDam, MinAtt, MinDef, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl) {
    if (LootDebug) Log.OutD("Loot_ModWeapon: Begin, Index [" + Index + "]", LootDebugScreen);
    if ((MinDam    == undefined) || (MinDam    == null)) MinDam    = -1;
    if ((MinDef    == undefined) || (MinDef    == null)) MinDef    = -1;
    if ((MinAtt    == undefined) || (MinAtt    == null)) MinAtt    = -1;
    if ((MaxWork   == undefined) || (MaxWork   == null)) MaxWork   = -1;
    if ((MaxBU     == undefined) || (MaxBU     == null)) MaxBU     = -1;
    if ((MaxValue  == undefined) || (MaxValue  == null)) MaxValue  = -1;
    if ((MinSpells == undefined) || (MinSpells == null)) MinSpells = -1;
    if ((MinSplLvl == undefined) || (MinSplLvl == null)) MinSplLvl = -1;

    if (MinDam    >=0) this.WeaponFilter[Index].MinDam    = MinDam;
    if (MinAtt    >=0) this.WeaponFilter[Index].MinAtt    = MinAtt;
    if (MinDef    >=0) this.WeaponFilter[Index].MinDef    = MinDef;
    if (MaxWork   >=0) this.WeaponFilter[Index].MaxWork   = MaxWork;
    if (MaxBU     >=0) this.WeaponFilter[Index].MaxBU     = MaxBU;
    if (MaxValue  >=0) this.WeaponFilter[Index].MaxValue  = MaxValue;
    if (MinSpells >=0) this.WeaponFilter[Index].MinSpells = MinSpells;
    if (MinSplLvl >=0) this.WeaponFilter[Index].MinSplLvl = MinSplLvl;
    if (LootDebug) Log.OutD("Loot_ModWeapon: End", LootDebugScreen);
} // end function

function Loot_ShowWeapon() {
    if (LootDebug) Log.OutD("Loot_ShowWeapon: Begin", LootDebugScreen);

    var Header = false;
    for (var x=0; x<this.WeaponFilter.length; x++) {
	with (this.WeaponFilter[x]) {
 	    if (!Enabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width(" ",1.8) +
			    Util.Width("Min",0.6) + Util.Width("Min",0.6) +
			    Util.Width("Min",0.6) +  Util.Width("Max",0.7) +
			    Util.Width("Max",0.7) +  Util.Width("Max",0.7) +
			    Util.Width("Min",0.6) + "MinSpl");

		Chat.Screen(Util.Width("Weapon",1.8) +
			    Util.Width("Dam",0.6) + Util.Width("Att",0.6) +
			    Util.Width("Def",0.6) + Util.Width("Work",0.7) +
			    Util.Width("BU",0.7)  + Util.Width("Val",0.7) +
			    Util.Width("Spl",0.6) + "Lvl");
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,1.8) +
	    		Util.Width(MinDam+" ",0.6)    + Util.Width(MinAtt+" ",0.6) +
	    		Util.Width(MinDef+" ",0.6)    + Util.Width(MaxWork+" ",0.7) +
			Util.Width(MaxBU+" ",0.7)     + Util.Width(MaxValue+ " ",0.7) +
			Util.Width(MinSpells+" ",0.6) + MinSplLvl);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowWeapon: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_ResetWeapon() {
    if (LootDebug) Log.OutD("Loot_ResetWeapon: Begin", LootDebugScreen);
    // Melee Weapons
    //
    // Axe	Max Dam 36
    // Dagger	Max Dam 15
    // Mace	Max Dam 33
    // Spear	Max Dam 28
    // Staff	Max Dam 15
    // Sword	Max Dam 40
    // UA	Max Dam 14
    //
    // Ranged Weapons.  Attack bonus meaningless on these weapons.
    //
    // Bow 	Max Dam Bonus 130
    // XBow	Max Dam Bonus 150
    // TW	Max Dam Bonus 150
    //					       Min   Min Min Max Max   Max    Min  Min Spl
    //                                  Enable Dam   Att Def Wrk BU    Value  Spl  Level
    this.SetWeapon("Bow",		true,  130,  0,  6,  7,  1500, 40000,  0,  6);
    this.SetWeapon("Crossbow",		true,  150,  0,  6,  7,  2500, 40000,  0,  6);
    this.SetWeapon("Sword (Pierce)",	true,   37,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Sword (S/P)",	true,   37,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Sword (Cold)",	true,   37,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Sword (Fire)",	true,   37,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Sword (Acid)",	true,   37,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Sword (Elec)",	true,   37,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("UA (Slash)",	true,   13,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("UA (S/P)",		true,   13,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("UA (Bludg)",	true,   13,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("UA (Cold)",		true,   13,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("UA (Fire)",		true,   13,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("UA (Acid)",		true,   13,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("UA (Elec)",		true,   13,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("Axe (Slash)",	false,  35,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Axe (Bludg)",	false,  35,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Axe (Cold)",	false,  35,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Axe (Fire)",	false,  35,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Axe (Acid)",	false,  35,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Axe (Elec)",	false,  35,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Dagger (S/P)",	false,  15,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("Dagger (Cold)",	false,  15,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("Dagger (Fire)",	false,  15,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("Dagger (Acid)",	false,  15,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("Dagger (Elec)",	false,  15,  7,  7,  6,  1000, 25000,  0,  6);
    this.SetWeapon("Mace (Bludg)",	false,  33,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Mace (Cold)",	false,  33,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Mace (Fire)",	false,  33,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Mace (Acid)",	false,  33,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Mace (Elec)",	false,  33,  7,  7,  6,  2000, 25000,  0,  6);
    this.SetWeapon("Spear (Pierce)",	false,  28,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Spear (S/P)",	false,  28,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Spear (Cold)",	false,  28,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Spear (Fire)",	false,  28,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Spear (Acid)",	false,  28,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Spear (Elec)",	false,  28,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Staff (Bludg)",	false,  15,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Staff (Cold)",	false,  15,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Staff (Fire)",	false,  15,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Staff (Acid)",	false,  15,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Staff (Elec)",	false,  15,  7,  7,  6,  1500, 25000,  0,  6);
    this.SetWeapon("Thrown Weapons",	false, 150,  0,  6,  6,  1500, 25000,  0,  6);
    if (LootDebug) Log.OutD("Loot_ResetWeapon: End", LootDebugScreen);
} // end func

function Loot_LookupWeapon(Name) {
    if (LootDebug2) Log.OutD("Loot_LookupWeapon: Begin, Name [" + Name + "]", LootDebugScreen);
    for (var x=0; x<this.WeaponFilter.length; x++) if (this.WeaponFilter[x].Name == Name) {
  	if (LootDebug2) Log.OutD("Loot_LookupWeapon: End, Found [" + x + "]", LootDebugScreen);
    	return x;
    } // endfor
    if (LootDebug2) Log.OutD("Loot_LookupWeapon: End, not found", LootDebugScreen);
    return -1;	  // not found
} // end func


// WAND METHODS ***********************************************************

function Loot_EnableWand(Work) {
    this.WandFilter[Work].Enabled = true;
} // end func

function Loot_DisableWand(Work) {
    this.WandFilter[Work].Enabled = false;
} // end func

function Loot_SetWand(Work, Enabled, MaxValue, MinManaC) {
    if ((Work<1) || (Work>10)) return false;
    this.WandFilter[Work].Enabled   = Enabled;
    this.WandFilter[Work].MaxValue  = MaxValue;
    this.WandFilter[Work].MinManaC  = MinManaC;
    this.WandFilter[Work].MinMeleeD = 0;
    this.WandFilter[Work].AndOrFlag = 0;
    return true;
} // end func

function Loot_SetWand2(Work, Enabled, MaxValue, MinManaC, MinMeleeD, AndOrFlag) {
    if ((Work<1) || (Work>10)) return false;
    this.WandFilter[Work].Enabled   = Enabled;
    this.WandFilter[Work].MaxValue  = MaxValue;
    this.WandFilter[Work].MinManaC  = MinManaC;
    this.WandFilter[Work].MinMeleeD = MinMeleeD;
    this.WandFilter[Work].AndOrFlag = AndOrFlag;
    return true;
} // end func

function Loot_ModWand(Work, MaxValue, MinManaC, MinMeleeD, AndOrFlag) {
    if ((MaxValue  == undefined) || (MaxValue  == null)) MaxValue  = -1;
    if ((MinManaC  == undefined) || (MinManaC  == null)) MinManaC  = -1;
    if ((MinMeleeD == undefined) || (MinMeleeD == null)) MinMeleeD = -1;
    if ((AndOrFlag == undefined) || (AndOrFlag == null)) AndOrFlag = -1;

    if (MaxValue  >= 0) this.WandFilter[Work].MaxValue  = MaxValue;
    if (MinManaC  >= 0) this.WandFilter[Work].MinManaC  = MinManaC;
    if (MinMeleeD >= 0) this.WandFilter[Work].MinMeleeD = MinMeleeD;
    if (AndOrFlag >= 0) this.WandFilter[Work].AndOrFlag = AndOrFlag;
} // end func

function Loot_ShowWand() {
    if (LootDebug) Log.OutD("Loot_ShowWand: Begin", LootDebugScreen);

    var Header = false;
    for (var x=1; x<11; x++) {
	with (this.WandFilter[x]) {
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width("Wand",1.8) + Util.Width("Max",0.9) + Util.Width("Min",0.9) + "Min");
		Chat.Screen(Util.Width("(Work)",0.8) + Util.Width("Enabled",1.0) + Util.Width("Value",0.9) +
				       Util.Width("ManaC",0.9) + Util.Width("MeleeD",0.9) + "Match" );
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(x + " ",0.8) + Util.Width(Enabled+" ",1.0) + Util.Width(MaxValue+" ",0.9) +
	    		Util.Width(MinManaC+" ",0.9) + Util.Width(MinMeleeD+" ",0.9) + andorFlagText[AndOrFlag] );
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowWeapon: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_ResetWand() {
    // Loot_SetWand2(Work, Enabled, MaxValue, MinManaC, MinMeleeD, AndOrFlag)
    this.SetWand2(  1, true,   20000,   8, 12, 1 );
    this.SetWand2(  2, true,   20000,   8, 12, 1 );
    this.SetWand2(  3, true,   20000,   8, 12, 1 );
    this.SetWand2(  4, true,   20000,   8, 12, 1 );
    this.SetWand2(  5, true,   12000,   8, 12, 1 );
    this.SetWand2(  6, true,   12000,  10, 13, 1 );
    this.SetWand2(  7, false,  12000,  10, 13, 1 );
    this.SetWand2(  8, false,  12000,  10, 13, 1 );
    this.SetWand2(  9, false,  12000,  10, 13, 1 );
    this.SetWand2( 10, false,  12000,  10, 13, 1 );
} // end func


// SALVAGE METHODS ***********************************************************

function Loot_EnableSalvage(salIndex) {
    this.SalvageFilter[salIndex].Enabled = true;
} // end func

function Loot_DisableSalvage(salIndex) {
    this.SalvageFilter[salIndex].Enabled = false;
} // end func

function Loot_AddSalvage(MaterialId, SalType, Name, Desc) {
    var NextSlot = this.SalvageFilter.length;
    this.SalvageFilter[NextSlot] = new SalvageFilterRec(MaterialId, SalType, Name, Desc);
    this.SalvageIndex[MaterialId] = NextSlot;
    this.SalvageOpts = this.SalvageOpts + " <option data='" + NextSlot + "' text='" + Name + "'/>\n";
} // end func

function Loot_SetSalvage(MaterialId, Enabled, MinWork, MaxBU, SortPrio, CombineMask, ExcludeVal) {
    if (this.SalvageIndex[MaterialId] == undefined) {
	Chat.Screen("SetSalvage: Error, material id [" + MaterialId + "] unknown!");
    	return false;
    } // endif
    salIndex = this.SalvageIndex[MaterialId];
    this.SalvageFilter[salIndex].Enabled	= Enabled;
    this.SalvageFilter[salIndex].ExcludeVal	= ExcludeVal;
    this.SalvageFilter[salIndex].MinWork	= MinWork;
    this.SalvageFilter[salIndex].MaxBU		= MaxBU;
    this.SalvageFilter[salIndex].SortPrio	= SortPrio;
    this.SalvageFilter[salIndex].CombineMask	= CombineMask;
    this.SetWorkRange(salIndex);
    return true;
} // end func

function Loot_ModSalvage(salIndex, MinWork, MaxBU, SortPrio, CombineMask, ExcludeVal) {
    if ((MinWork == undefined) || (MinWork == null)) MinWork = -1;
    if ((MaxBU == undefined) || (MaxBU == null)) MaxBU = -1;
    if ((SortPrio == undefined) || (SortPrio == null)) SortPrio = -1;
    if ((CombineMask == undefined) || (CombineMask == null)) CombineMask = -1;

    if ((ExcludeVal != undefined) && (ExcludeVal != null)) {
    	this.SalvageFilter[salIndex].ExcludeVal = ExcludeVal;
    } // endif

    if (MinWork >= 0)	this.SalvageFilter[salIndex].MinWork	= MinWork;
    if (MaxBU  >= 0)	this.SalvageFilter[salIndex].MaxBU	= MaxBU;
    if (SortPrio >= 0)	this.SalvageFilter[salIndex].SortPrio	= SortPrio;
    if (CombineMask  >= 0) {
    	this.SalvageFilter[salIndex].CombineMask = CombineMask;
    	this.SetWorkRange(salIndex);
    } // endif
} // end func

function Loot_ResetSalvage() {
   // Armor Tinkering
    this.SetSalvage(materialSteel,	    true,	5, this.SalDefBur,   7200,  combineMaskVal[7]);
    this.SetSalvage(materialAlabaster,	    true,	5, this.SalDefBur,   7100,  combineMaskVal[7]);
    this.SetSalvage(materialBronze,	    true,	5, this.SalDefBur,   7000,  combineMaskVal[7]);
    this.SetSalvage(materialMarble,	    true,	5, this.SalDefBur,   6900,  combineMaskVal[7]);
    this.SetSalvage(materialCeramic,	    true,	5, this.SalDefBur,   6800,  combineMaskVal[7]);
    this.SetSalvage(materialDilloHide,	    true,	5, this.SalDefBur,   6700,  combineMaskVal[7]);
    this.SetSalvage(materialReedsharkHide,  true,	5, this.SalDefBur,   6600,  combineMaskVal[7]);
    this.SetSalvage(materialWool,	    true,	5, this.SalDefBur,   6500,  combineMaskVal[7]);

    // Weapon Tinkering
    this.SetSalvage(materialBrass,	    true,	5, this.SalDefBur,   6400,  combineMaskVal[7]);
    this.SetSalvage(materialGranite,	    true,	4, this.SalDefBur,   6300,  combineMaskVal[7]);
    this.SetSalvage(materialIron,	    true,	5, this.SalDefBur,   6200,  combineMaskVal[7]);
    this.SetSalvage(materialMahogany,	    true,	4, this.SalDefBur,   6100,  combineMaskVal[7]);
    this.SetSalvage(materialOak,	    true,	5, this.SalDefBur,   6000,  combineMaskVal[7]);
    this.SetSalvage(materialVelvet,	    true,	5, this.SalDefBur,   5900,  combineMaskVal[7]);

    // Item Tinkering
    this.SetSalvage(materialGold,	    true,	5, this.SalDefBur,   5800,  combineMaskVal[7]);
    this.SetSalvage(materialLinen,	    true,	5, this.SalDefBur,   5700,  combineMaskVal[7]);
    this.SetSalvage(materialPine,	    true,	5, this.SalDefBur,   5600,  combineMaskVal[7]);
    this.SetSalvage(materialMoonstone,	    true,	5, this.SalDefBur,   5500,  combineMaskVal[7]);
    this.SetSalvage(materialIvory,	    true,	5, this.SalDefBur,   5400,  combineMaskVal[10]);
    this.SetSalvage(materialLeather,	    true,	5, this.SalDefBur,   5300,  combineMaskVal[10]);

    // Magic Item Tinkering
    this.SetSalvage(materialBlackOpal,	    true,	5, this.SalDefBur,   5200,  combineMaskVal[10]);
    this.SetSalvage(materialFireOpal,	    true,	5, this.SalDefBur,   5100,  combineMaskVal[10]);
    this.SetSalvage(materialSunstone,	    true,	5, this.SalDefBur,   5000,  combineMaskVal[10]);
    this.SetSalvage(materialOpal,	    true,	5, this.SalDefBur,   4900,  combineMaskVal[7]);

    // Rendering
    this.SetSalvage(materialWhiteSapphire,  true,	5, this.SalDefBur,   4800,  combineMaskVal[10]);
    this.SetSalvage(materialBlackGarnet,    true,	4, this.SalDefBur,   4700,  combineMaskVal[10]);
    this.SetSalvage(materialImperialTopaz,  true,	5, this.SalDefBur,   4600,  combineMaskVal[10]);
    this.SetSalvage(materialEmerald,	    true,	5, this.SalDefBur,   4500,  combineMaskVal[10]);
    this.SetSalvage(materialRedGarnet,      true,	4, this.SalDefBur,   4400,  combineMaskVal[10]);
    this.SetSalvage(materialAquamarine,     true,	5, this.SalDefBur,   4300,  combineMaskVal[10]);
    this.SetSalvage(materialJet,	    true,	5, this.SalDefBur,   4200,  combineMaskVal[10]);

    // Requirements
    this.SetSalvage(materialEbony,	    true,	5, this.SalDefBur,   4100,  combineMaskVal[10]);
    this.SetSalvage(materialTeak,	    true,	5, this.SalDefBur,   4000,  combineMaskVal[10]);
    this.SetSalvage(materialPorcelain,	    true,	5, this.SalDefBur,   3900,  combineMaskVal[10]);
    this.SetSalvage(materialSilver,	    true,	5, this.SalDefBur,   3800,  combineMaskVal[10]);
    this.SetSalvage(materialCopper,	    true,	5, this.SalDefBur,   3700,  combineMaskVal[10]);
    this.SetSalvage(materialSilk,	    true,	5, this.SalDefBur,   3600,  combineMaskVal[10]);

    // Armor Imbues
    this.SetSalvage(materialYellowTopaz,    true,	5, this.SalDefBur,   3500,  combineMaskVal[10]);
    this.SetSalvage(materialPeridot,	    true,	5, this.SalDefBur,   3400,  combineMaskVal[10]);
    this.SetSalvage(materialZircon,	    true,	5, this.SalDefBur,   3300,  combineMaskVal[10]);

    // Jewelry Imbues
    this.SetSalvage(materialLavenderJade,   true,	5, this.SalDefBur,   3200,  combineMaskVal[10]);
    this.SetSalvage(materialRedJade,	    true,	5, this.SalDefBur,   3100,  combineMaskVal[10]);
    this.SetSalvage(materialCitrine,	    true,	5, this.SalDefBur,   3000,  combineMaskVal[10]);
    this.SetSalvage(materialCarnelian,	    true,	5, this.SalDefBur,   2900,  combineMaskVal[10]);
    this.SetSalvage(materialRoseQuartz,	    true,	5, this.SalDefBur,   2800,  combineMaskVal[10]);
    this.SetSalvage(materialBloodstone,	    true,	5, this.SalDefBur,   2700,  combineMaskVal[10]);
    this.SetSalvage(materialSmokeyQuartz,   true,	5, this.SalDefBur,   2600,  combineMaskVal[10]);
    this.SetSalvage(materialAgate,	    true,	5, this.SalDefBur,   2500,  combineMaskVal[10]);
    this.SetSalvage(materialLapisLazuli,    true,	5, this.SalDefBur,   2400,  combineMaskVal[10]);
    this.SetSalvage(materialMalachite,	    true,	5, this.SalDefBur,   2300,  combineMaskVal[10]);
    this.SetSalvage(materialHematite,	    true,	5, this.SalDefBur,   2200,  combineMaskVal[10]);
    this.SetSalvage(materialAzurite,	    true,	5, this.SalDefBur,   2100,  combineMaskVal[10]);

    // Unknown Use
    this.SetSalvage(materialAmber,	    false,	5, this.SalDefBur,   2000,  combineMaskVal[10]);
    this.SetSalvage(materialAmethyst,	    false,	5, this.SalDefBur,   1900,  combineMaskVal[10]);
    this.SetSalvage(materialDiamond,	    false,	5, this.SalDefBur,   1800,  combineMaskVal[10]);
    this.SetSalvage(materialGreenGarnet,    false,	5, this.SalDefBur,   1700,  combineMaskVal[10]);
    this.SetSalvage(materialGreenJade,	    false,	5, this.SalDefBur,   1600,  combineMaskVal[10]);
    this.SetSalvage(materialGromnieHide,    false,	5, this.SalDefBur,   1500,  combineMaskVal[10]);
    this.SetSalvage(materialObsidian,	    false,	5, this.SalDefBur,   1400,  combineMaskVal[10]);
    this.SetSalvage(materialOnyx,	    false,	5, this.SalDefBur,   1300,  combineMaskVal[10]);
    this.SetSalvage(materialPyreal,	    false,	5, this.SalDefBur,   1200,  combineMaskVal[10]);
    this.SetSalvage(materialRuby,	    false,	5, this.SalDefBur,   1100,  combineMaskVal[10]);
    this.SetSalvage(materialSandstone,	    false,	5, this.SalDefBur,   1000,  combineMaskVal[10]);
    this.SetSalvage(materialSapphire,	    false,	5, this.SalDefBur,    900,  combineMaskVal[10]);
    this.SetSalvage(materialSatin,	    false,	5, this.SalDefBur,    800,  combineMaskVal[10]);
    this.SetSalvage(materialSerpentine,	    false,	5, this.SalDefBur,    700,  combineMaskVal[10]);
    this.SetSalvage(materialTigerEye,	    false,	5, this.SalDefBur,    600,  combineMaskVal[10]);
    this.SetSalvage(materialTourmaline,	    false,	5, this.SalDefBur,    500,  combineMaskVal[10]);
    this.SetSalvage(materialTurquoise,	    false,	5, this.SalDefBur,    400,  combineMaskVal[10]);
    this.SetSalvage(materialWhiteJade,	    false,	5, this.SalDefBur,    300,  combineMaskVal[10]);
    this.SetSalvage(materialWhiteQuartz,    false,	5, this.SalDefBur,    200,  combineMaskVal[10]);
    this.SetSalvage(materialYellowGarnet,   false,	5, this.SalDefBur,    100,  combineMaskVal[10]);
} // end func

function Loot_ShowSalvage(EnableFlag) {
   if ((EnableFlag==undefined) || (EnableFlag==null)) EnableFlag = false;
    if (LootDebug) Log.OutD("Loot_ShowSalvage: Begin, EnableFlag [" + EnableFlag + "]", LootDebugScreen);

   var Header = false;
    for (var x=0; x<this.SalvageFilter.length; x++) {
	with (this.SalvageFilter[x]) {
 	    if (EnableFlag && !Enabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width(" ",2.8) +
			    Util.Width("Exclude",1.0) + Util.Width("Min",0.6)  +
			    Util.Width("Max",0.6)     + Util.Width("Sort",0.6) +
			    "Combine" )

		Chat.Screen(Util.Width("Salvage",1.8) + Util.Width("Enabled",1.0) +
			    Util.Width("ValSalv",1.0) + Util.Width("Work",0.6)    +
			    Util.Width("BU",0.6)      + Util.Width("Prio",0.6)    +
			    "Points" );
	    } // endif

 	    CombPoints = "";
	    for (var y=1; y<11; y++)
		if (CombineMask & combineMaskVal[y]) CombPoints += y + "  ";

	    // Print info
	    Chat.Screen(Util.Width(Name+" ",1.8)       + Util.Width(Enabled+" ",1.0)  +
	    		Util.Width(ExcludeVal+" ",1.0) + Util.Width(MinWork+" ",0.6)  +
	    		Util.Width(MaxBU+" ",0.6)      + Util.Width(SortPrio+" ",0.6) +
	    		CombPoints);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowSalvage: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_SetWorkRange(salIndex) {
    with (this.SalvageFilter[salIndex]) {
	var LowIndex = 1;
	for (var x=1; x<=salvageMaxWork; x++) {
	    if (CombineMask & combineMaskVal[x]) {
	    	var HighIndex = x;
	    	if (LootDebug2) Log.OutD("SalvageInit: LowIndex [" + LowIndex + "] HighIndex [" + HighIndex + "]");
	    	for (var y=LowIndex; y<=HighIndex; y++) {
	    	    WorkRange[y] = new MaterialRangeRec(LowIndex, HighIndex+1);
	    	    if (LootDebug2) {
	    	    	Log.OutD("Loot_SetWorkRange: Work [" + y + "] Low [" + LowIndex + "] High [" + HighIndex + "]");
	    	    	Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + y + "].Low [" +
	    	    		  WorkRange[y].Low + "]");
	    	    	Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + y + "].High [" +
	    	    		  WorkRange[y].High + "]");
		    }// endif
	    	} // end for
	    	LowIndex = HighIndex + 1;
	    } // endif
	} // end for

	// Now define the records for the works above the highest specified.
	for (var x=LowIndex; x<=salvageMaxWork; x++) {
	    WorkRange[x] = new MaterialRangeRec(x,x+1);
	    if (LootDebug2) {
		Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + x + "].Low [" +
			  WorkRange[x].Low + "]");
		Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + x + "].High [" +
			  WorkRange[x].High + "]");
	    }// endif
	} // endfor
    } // end with
} // end func


// MISC FILTER MODIFICATION METHODS *******************************************

function Loot_ModValSalv(MinRatio, MaxBagValue) {
    if ((MinRatio    == undefined) || (MinRatio    == null)) MinRatio = -1;
    if ((MaxBagValue == undefined) || (MaxBagValue == null)) MaxBagValue = -1;
    if (MinRatio    >= 0) this.ValSalvFilter.MinRatio	 = MinRatio;
    if (MaxBagValue >= 0) this.ValSalvFilter.MaxBagValue = MaxBagValue;
} // end func

function Loot_ShowValSalv() {
} // end func

function Loot_ResetValSalv() {
    this.ModValSalv(0, 100000);
} // end func
    
    
function Loot_ModJewelry(MinSplLvl1, MinSplLvl2, MinSplLvl3) {
    if ((MinSplLvl1 == undefined) || (MinSplLvl1 == null)) MinSplLvl1 = -1;
    if ((MinSplLvl2 == undefined) || (MinSplLvl2 == null)) MinSplLvl2 = -1;
    if ((MinSplLvl3 == undefined) || (MinSplLvl3 == null)) MinSplLvl3 = -1;
    if (MinSplLvl1 >= 0) this.JewelryFilter.MinSplLvl1 = MinSplLvl1;
    if (MinSplLvl2 >= 0) this.JewelryFilter.MinSplLvl2 = MinSplLvl2;
    if (MinSplLvl3 >= 0) this.JewelryFilter.MinSplLvl3 = MinSplLvl3;
} // end func

function Loot_ShowJewelry() {
} // end func

function Loot_ResetJewelry() {
    this.ModJewelry(6, 6);
} // end func


function Loot_ModClothing(MinSplLvl1, MinSplLvl2, MinSplLvl3) {
    if ((MinSplLvl1 == undefined) || (MinSplLvl1 == null)) MinSplLvl1 = -1;
    if ((MinSplLvl2 == undefined) || (MinSplLvl2 == null)) MinSplLvl2 = -1;
    if ((MinSplLvl3 == undefined) || (MinSplLvl3 == null)) MinSplLvl3 = -1;

    if (MinSplLvl1 >= 0) this.ClothingFilter.MinSplLvl1 = MinSplLvl1;
    if (MinSplLvl2 >= 0) this.ClothingFilter.MinSplLvl2 = MinSplLvl2;
    if (MinSplLvl3 >= 0) this.ClothingFilter.MinSplLvl3 = MinSplLvl3;
} // end func

function Loot_ShowClothing() {
} // end func

function Loot_ResetClothing() {
    this.ModClothing(6, 6);
} // end func


function Loot_ModScroll(MinLvl, MaxLvl, UnknownOnly, LearnUnknown) {
    if ((MinLvl       == undefined) || (MinLvl       == null)) MinLvl = -1;
    if ((MaxLvl       == undefined) || (MaxLvl       == null)) MaxLvl = -1

    if (MinLvl >= 0) this.ScrollFilter.MinLvl = MinLvl;
    if (MaxLvl >= 0) this.ScrollFilter.MaxLvl = MaxLvl;
    if ((UnknownOnly  != undefined) && (UnknownOnly  != null)) this.ScrollFilter.UnknownOnly  = UnknownOnly;
    if ((LearnUnknown != undefined) && (LearnUnknown != null)) this.ScrollFilter.LearnUnknown = LearnUnknown;
} // end func

function Loot_ShowScroll() {
} // end func

function Loot_ResetScroll() {
    this.ModScroll(7, 7, false, false);
} // end func


function Loot_ModValuable(MaxBU, MinValue, MaxTotal) {
    if ((MaxBU    == undefined) || (MaxBU     == null)) MaxBU = -1;
    if ((MinValue == undefined) || (MinValue  == null)) MinValue  = -1;
    if ((MaxTotal == undefined) || (MaxTotal  == null)) MaxTotal  = -1;

    if (MaxBU    >= 0) this.ValuableFilter.MaxBU    = MaxBU;
    if (MinValue >= 0) this.ValuableFilter.MinValue = MinValue ;
    if (MaxTotal >= 0) this.ValuableFilter.MaxTotal = MaxTotal ;
} // end func

function Loot_ShowValuable() {
} // end func

function Loot_ResetValuable() {
    this.ModValuable(250, 8000, 9999999);
} // end func


Loot = new LootObj();


// For reference, here are all the equipment slots.
// var eqmNil                      =              0;	// No slot.
// var eqmHead                     =     0x00000001;
// var eqmChestUnder               =     0x00000002;
// var eqmAbdomenUnder             =     0x00000004;
// var eqmUpperArmsUnder           =     0x00000008;
// var eqmLowerArmsUnder           =     0x00000010;
// var eqmHands                    =     0x00000020;
// var eqmUpperLegsUnder           =     0x00000040;
// var eqmLowerLegsUnder           =     0x00000080;
// var eqmFeet                     =     0x00000100;
// var eqmChestOuter               =     0x00000200;
// var eqmAbdomenOuter             =     0x00000400;
// var eqmUpperArmsOuter           =     0x00000800;
// var eqmLowerArmsOuter           =     0x00001000;
// var eqmUpperLegsOuter           =     0x00002000;
// var eqmLowerLegsOuter           =     0x00004000;
// var eqmNecklace                 =     0x00008000;
// var eqmRBracelet                =     0x00010000;
// var eqmLBracelet                =     0x00020000;
// var eqmBracelets                =     0x00030000;	// Both bracelet slots.
// var eqmRRing                    =     0x00040000;
// var eqmLRing                    =     0x00080000;
// var eqmRings                    =     0x000C0000;	// Both ring slots.
// var eqmWeapon                   =     0x00100000;	// Melee weapon slot.
// var eqmShield                   =     0x00200000;
// var eqmRangedWeapon             =     0x00400000;	// Bow or crossbow slot.
// var eqmAmmo                     =     0x00800000;
// var eqmFocusWeapon              =     0x01000000;	// Wand, staff, or orb slot.
// var eqmAll                      =     0x01FFFFFF;	// All equipment slots.
         